@@ -6,6 +6,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -27,8 +35,6 @@
 #include <unistd.h>
 
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-authentication-manager.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <gst/gst.h>
 
 #include <dbus/dbus.h>
@@ -487,8 +493,6 @@ main (int argc, char **argv)
 	textdomain (GETTEXT_PACKAGE);
 #endif
 	g_type_init ();
-	gnome_vfs_init ();
-	gnome_authentication_manager_init ();
 	gst_init (NULL, NULL);
 	g_set_prgname ("rhythmbox-metadata");
 
@@ -551,7 +555,7 @@ main (int argc, char **argv)
 				       g_main_loop_get_context (svc.loop));
 
 	if (!svc.external)
-		g_timeout_add (ATTENTION_SPAN * 500, (GSourceFunc) electromagnetic_shotgun, &svc);
+		g_timeout_add_seconds (ATTENTION_SPAN / 2, (GSourceFunc) electromagnetic_shotgun, &svc);
 
 	g_main_loop_run (svc.loop);
 
