@@ -8,7 +8,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -34,8 +34,6 @@
 
 G_BEGIN_DECLS
 
-#define RB_METADATA_NUM_FIELDS 23
-
 typedef enum
 {
 	RB_METADATA_FIELD_TITLE,                   /* string */
@@ -75,7 +73,7 @@ typedef enum
 
 typedef enum
 {
-	RB_METADATA_ERROR_GNOMEVFS,
+	RB_METADATA_ERROR_IO,
 	RB_METADATA_ERROR_MISSING_PLUGIN,
 	RB_METADATA_ERROR_UNRECOGNIZED,
 	RB_METADATA_ERROR_UNSUPPORTED,
