@@ -2,13 +2,21 @@
  *
  *  arch-tag: Implementation of audiocd source object (based of the ipod source)
  *
- *  Copyright (C) 2005-2006 James Livingston  <jrl@ids.org.au>
+ *  Copyright (C) 2005-2006 James Livingston  <doclivingston@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -30,22 +38,21 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <gst/gst.h>
-#include <totem-disc.h>
+#include <gst/cdda/gstcddabasesrc.h>
 
 #include "rb-plugin.h"
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
+#include "rb-shell.h"
 #include "rb-audiocd-source.h"
 #include "rb-util.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
 #include "rb-glade-helpers.h"
 
-#ifdef HAVE_MUSICBRAINZ
-#include "sj-metadata-musicbrainz.h"
+#ifdef HAVE_SJ_METADATA_GETTER
+#include "sj-metadata-getter.h"
 #include "sj-structures.h"
 #endif
 
@@ -74,15 +81,15 @@ typedef struct
 	GstElement *cdda;
 	GstElement *fakesink;
 
-#ifdef HAVE_MUSICBRAINZ
-	SjMetadata *metadata;
+#ifdef HAVE_SJ_METADATA_GETTER
+	SjMetadataGetter *metadata;
 #endif
 } RBAudioCdSourcePrivate;
 
 RB_PLUGIN_DEFINE_TYPE (RBAudioCdSource, rb_audiocd_source, RB_TYPE_REMOVABLE_MEDIA_SOURCE)
 #define AUDIOCD_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_AUDIOCD_SOURCE, RBAudioCdSourcePrivate))
 
-#ifdef HAVE_MUSICBRAINZ
+#ifdef HAVE_SJ_METADATA_GETTER
 static AlbumDetails* multiple_album_dialog (GList *albums, RBAudioCdSource *source);
 #endif
 static void
@@ -95,7 +102,6 @@ rb_audiocd_source_class_init (RBAudioCdSourceClass *klass)
 	object_class->dispose = rb_audiocd_source_dispose;
 
 	/* don't bother showing the browser/search bits */
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
@@ -122,10 +128,8 @@ rb_audiocd_source_dispose (GObject *object)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (object);
 
-	if (priv->device_path) {
-		g_free (priv->device_path);
-		priv->device_path = NULL;
-	}
+	g_free (priv->device_path);
+
 	if (priv->tracks) {
 		g_list_free (priv->tracks);
 		priv->tracks = NULL;
@@ -164,33 +168,28 @@ rb_audiocd_source_constructor (GType type,
 RBRemovableMediaSource *
 rb_audiocd_source_new (RBPlugin *plugin,
 		       RBShell *shell,
-		       GnomeVFSVolume *volume)
+		       GVolume *volume)
 {
-	char *device_path;
 	GObject *source;
 	RhythmDBEntryType entry_type;
 	RhythmDB *db;
 	char *name;
 	char *path;
 
-	if (!rb_audiocd_is_volume_audiocd (volume))
-		return NULL;
+	g_object_get (shell, "db", &db, NULL);
 
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-	path = gnome_vfs_volume_get_device_path (volume);
+	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	name = g_strdup_printf ("audiocd: %s", path);
 	entry_type = rhythmdb_entry_register_type (db, name);
-	g_object_unref (G_OBJECT (db));
 	g_free (name);
 	g_free (path);
+	g_object_unref (db);
 
 	entry_type->category = RHYTHMDB_ENTRY_NORMAL;
 	entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc)rb_true_function;
-	/* TODO same the metadata somewhere */
+	/* TODO save the metadata somewhere */
 	entry_type->sync_metadata = (RhythmDBEntrySyncFunc)rb_null_function;
 
-	device_path = gnome_vfs_volume_get_device_path (volume);
-
 	source = g_object_new (RB_TYPE_AUDIOCD_SOURCE,
 			       "entry-type", entry_type,
 			       "volume", volume,
@@ -200,8 +199,6 @@ rb_audiocd_source_new (RBPlugin *plugin,
 			       "plugin", plugin,
 			       NULL);
 
-	g_free (device_path);
-
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
 
 	return RB_REMOVABLE_MEDIA_SOURCE (source);
@@ -241,6 +238,7 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 	RhythmDBEntry *entry;
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	char *audio_path;
+	guint64 duration;
 	GValue value = {0, };
 	gchar *str;
 	RhythmDBEntryType entry_type;
@@ -273,27 +271,17 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 			    &value);
 	g_value_unset (&value);
 
-	/* determine the duration */
-	{
-		GstFormat time_format = GST_FORMAT_TIME;
-		GstFormat track_format = gst_format_get_by_nick ("track");
-		gint64 duration;
-		gboolean result;
-                result = gst_element_seek (priv->fakesink, 1.0, track_format, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, (guint64) track_number - 1, GST_SEEK_TYPE_NONE, -1);
-		if (result) {
-			result = gst_element_query_duration (priv->fakesink, &time_format, &duration) && time_format == GST_FORMAT_TIME;
-		}
-
-		if (result) {
-			g_value_init (&value, G_TYPE_ULONG);
-			g_value_set_ulong (&value, (gulong)(duration / GST_SECOND));
-			rhythmdb_entry_set (db, entry,
-					    RHYTHMDB_PROP_DURATION,
-					    &value);
-			g_value_unset (&value);
-		} else {
-			g_warning ("Failed to query cd track duration");
-		}
+	/* determine the duration
+	 * FIXME: http://bugzilla.gnome.org/show_bug.cgi?id=551011 */
+	if (gst_tag_list_get_uint64 (GST_CDDA_BASE_SRC(priv->cdda)->tracks[track_number - 1].tags, GST_TAG_DURATION, &duration)) {
+		g_value_init (&value, G_TYPE_ULONG);
+		g_value_set_ulong (&value, (gulong)(duration / GST_SECOND));
+		rhythmdb_entry_set (db, entry,
+				    RHYTHMDB_PROP_DURATION,
+				    &value);
+		g_value_unset (&value);
+	} else {
+		g_warning ("Failed to query cd track duration");
 	}
 
 	entry_set_string_prop (db, entry, FALSE, RHYTHMDB_PROP_ARTIST, NULL);
@@ -383,9 +371,9 @@ get_db_for_source (RBAudioCdSource *source)
         return db;
 }
 
-#ifdef HAVE_MUSICBRAINZ
+#ifdef HAVE_SJ_METADATA_GETTER
 
-/**
+/*
  * Called by the Multiple Album dialog when the user hits return in
  * the list view
  */
@@ -400,7 +388,7 @@ album_row_activated (GtkTreeView *treeview,
 	gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
-/**
+/*
  * Utility function for when there are more than one albums
  * available. Borrowed from Sound Juicer.
  */
@@ -501,7 +489,7 @@ multiple_album_dialog (GList *albums, RBAudioCdSource *source)
 
 
 static void
-metadata_cb (SjMetadata *metadata,
+metadata_cb (SjMetadataGetter *metadata,
 	     GList *albums,
 	     GError *error,
 	     RBAudioCdSource *source)
@@ -517,7 +505,6 @@ metadata_cb (SjMetadata *metadata,
 	if (error != NULL) {
 		rb_debug ("Failed to load cd metadata: %s", error->message);
 		/* TODO display error to user? */
-		g_error_free (error);
 		g_object_unref (metadata);
 		return;
 	}
@@ -544,6 +531,13 @@ metadata_cb (SjMetadata *metadata,
 	} else
 		album = (AlbumDetails *)albums->data;
 
+	if (album->metadata_source == SOURCE_FALLBACK) {
+		g_object_unref (metadata);
+		priv->metadata = NULL;
+		g_object_unref (db);
+		return;
+	}
+
 	g_object_set (G_OBJECT (source), "name", album->title, NULL);
 
 	while (album->tracks && cd_track) {
@@ -611,7 +605,7 @@ metadata_cb (SjMetadata *metadata,
 }
 
 static void
-metadata_cancelled_cb (SjMetadata *metadata,
+metadata_cancelled_cb (SjMetadataGetter *metadata,
 		       GList *albums,
 		       GError *error,
 		       gpointer old_source)
@@ -625,22 +619,22 @@ static void
 rb_audiocd_load_metadata (RBAudioCdSource *source,
 			  RhythmDB *db)
 {
-#ifdef HAVE_MUSICBRAINZ
+#ifdef HAVE_SJ_METADATA_GETTER
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
-	priv->metadata = (SjMetadata*)sj_metadata_musicbrainz_new ();
-	sj_metadata_set_cdrom (priv->metadata, priv->device_path);
+	priv->metadata = sj_metadata_getter_new ();
+	sj_metadata_getter_set_cdrom (priv->metadata, priv->device_path);
 
 	g_signal_connect (G_OBJECT (priv->metadata), "metadata",
 			  G_CALLBACK (metadata_cb), source);
-	sj_metadata_list_albums (priv->metadata, NULL);
+	sj_metadata_getter_list_albums (priv->metadata, NULL);
 #endif
 }
 
 static void
 rb_audiocd_load_metadata_cancel (RBAudioCdSource *source)
 {
-#ifdef HAVE_MUSICBRAINZ
+#ifdef HAVE_SJ_METADATA_GETTER
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
 	if (priv->metadata) {
@@ -657,10 +651,11 @@ rb_audiocd_load_songs (RBAudioCdSource *source)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	RhythmDB *db;
-	GnomeVFSVolume *volume;
+	GVolume *volume;
 
 	g_object_get (source, "volume", &volume, NULL);
-	priv->device_path = gnome_vfs_volume_get_device_path (volume);
+	priv->device_path = g_volume_get_identifier (volume,
+						     G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	g_object_unref (volume);
 
 	db = get_db_for_source (source);
@@ -715,36 +710,34 @@ impl_delete_thyself (RBSource *source)
 }
 
 gboolean
-rb_audiocd_is_volume_audiocd (GnomeVFSVolume *volume)
+rb_audiocd_is_mount_audiocd (GMount *mount)
 {
-	char *device_path;
-	GnomeVFSDeviceType device_type;
 	gboolean result = FALSE;
+#if GLIB_CHECK_VERSION(2,17,7)
+	char **types;
+	guint i;
+	GError *error = NULL;
 
-	device_type = gnome_vfs_volume_get_device_type (volume);
-	device_path = gnome_vfs_volume_get_device_path (volume);
-
-	if (device_path == NULL)
-		return FALSE;
-
-	/* for sometimes device_type is GNOME_VFS_DEVICE_TYPE_CDROM */
-	if (device_type == GNOME_VFS_DEVICE_TYPE_AUDIO_CD || device_type == GNOME_VFS_DEVICE_TYPE_CDROM) {
-		GError *error = NULL;
-		MediaType media_type;
-
-		media_type = totem_cd_detect_type (device_path, &error);
-		g_free (device_path);
-		if (error != NULL) {
-			rb_debug ("error while detecting cd: %s", error->message);
-			g_error_free (error);
-			return FALSE;
+	types = g_mount_guess_content_type_sync (mount, FALSE, NULL, &error);
+	if (types == NULL) {
+		rb_debug ("error guessing content type: %s", error->message);
+		g_clear_error (&error);
+	} else {
+		for (i = 0; types[i] != NULL; i++) {
+			if (g_str_equal (types[i], "x-content/audio-cdda") != FALSE) {
+				result = TRUE;
+				break;
+			}
 		}
-		rb_debug ("detecting new cd - totem cd media type=%d", media_type);
-		return (media_type == MEDIA_TYPE_CDDA);
+		g_strfreev (types);
 	}
-
-	g_free (device_path);
-
+#else
+	GFile *file;
+	
+	file = g_mount_get_root (mount);
+	result = g_file_has_uri_scheme (file, "cdda");
+	g_object_unref (file);
+#endif /* glib 2.17.7 */
 	return result;
 }
 
@@ -760,9 +753,7 @@ impl_get_ui_actions (RBSource *source)
 {
 	GList *actions = NULL;
 
-#ifdef ENABLE_TRACK_TRANSFER
 	actions = g_list_prepend (actions, g_strdup ("RemovableSourceCopyAllTracks"));
-#endif
 	actions = g_list_prepend (actions, g_strdup ("RemovableSourceEject"));
 
 	return actions;
@@ -771,14 +762,18 @@ impl_get_ui_actions (RBSource *source)
 static guint
 impl_want_uri (RBSource *source, const char *uri)
 {
-	GnomeVFSVolume *volume;
-	char *activation_uri;
+	GVolume *volume;
+	GMount *mount;
+	GFile *file;
 	int retval;
 
 	retval = 0;
 
-	if (g_str_has_prefix (uri, "cdda://") == FALSE)
+	file = g_file_new_for_uri (uri);
+	if (g_file_has_uri_scheme (file, "cdda") == FALSE) {
+		g_object_unref (file);
 		return 0;
+	}
 
 	g_object_get (G_OBJECT (source),
 		      "volume", &volume,
@@ -786,14 +781,16 @@ impl_want_uri (RBSource *source, const char *uri)
 	if (volume == NULL)
 		return 0;
 
-	activation_uri = gnome_vfs_volume_get_activation_uri (volume);
-	if (activation_uri == NULL)
-		return 0;
+	mount = g_volume_get_mount (volume);
+	if (mount) {
+		GFile *root;
 
-	if (strcmp (activation_uri, uri) == 0)
-		retval = 100;
-
-	g_free (activation_uri);
+		root = g_mount_get_root (mount);
+		retval = g_file_equal (root, file) ? 100 : 0;
+		g_object_unref (mount);
+		g_object_unref (root);
+	}
+	g_object_unref (file);
 
 	return retval;
 }
