@@ -8,7 +8,7 @@
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
  * GStreamer plugins to be used and distributed together with GStreamer
  * and Rhythmbox. This permission is above and beyond the permissions granted
  * by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -37,23 +37,18 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 
-#ifdef HAVE_AVAHI_0_6
 #include <avahi-client/lookup.h>
 #include <avahi-client/publish.h>
-#endif
 #include <avahi-client/client.h>
 #include <avahi-common/error.h>
 #include <avahi-glib/glib-malloc.h>
 #include <avahi-glib/glib-watch.h>
 
+#include "rb-daap-mdns-avahi.h"
 #include "rb-daap-mdns-browser.h"
 #include "rb-marshal.h"
 #include "rb-debug.h"
 
-#ifdef HAVE_AVAHI_0_5
-#define AVAHI_ADDRESS_STR_MAX	(40)	/* IPv6 Max = 4*8 + 7 + 1 for NUL */
-#endif
-
 static void	rb_daap_mdns_browser_class_init (RBDaapMdnsBrowserClass *klass);
 static void	rb_daap_mdns_browser_init	(RBDaapMdnsBrowser	*browser);
 static void	rb_daap_mdns_browser_finalize   (GObject	        *object);
@@ -63,7 +58,6 @@ static void	rb_daap_mdns_browser_finalize   (GObject	        *object);
 struct RBDaapMdnsBrowserPrivate
 {
 	AvahiClient         *client;
-	AvahiGLibPoll       *poll;
 	AvahiServiceBrowser *service_browser;
 
 	GSList              *resolvers;
@@ -96,58 +90,6 @@ rb_daap_mdns_browser_error_quark (void)
 	return quark;
 }
 
-static void
-client_cb (AvahiClient         *client,
-	   AvahiClientState     state,
-	   RBDaapMdnsBrowser   *browser)
-{
-	/* Called whenever the client or server state changes */
-
-	switch (state) {
-#ifdef HAVE_AVAHI_0_6
-	case AVAHI_CLIENT_FAILURE:
-
-		 g_warning ("Client failure: %s\n", avahi_strerror (avahi_client_errno (client)));
-		 break;
-#endif
-	default:
-		break;
-	}
-}
-
-static void
-avahi_client_init (RBDaapMdnsBrowser *browser)
-{
-	int error = 0;
-
-	avahi_set_allocator (avahi_glib_allocator ());
-
-	browser->priv->poll = avahi_glib_poll_new (NULL, G_PRIORITY_DEFAULT);
-
-	if (! browser->priv->poll) {
-		rb_debug ("Unable to create AvahiGlibPoll object for mDNS");
-	}
-
-#ifdef HAVE_AVAHI_0_5
-	browser->priv->client = avahi_client_new (avahi_glib_poll_get (browser->priv->poll),
-						  (AvahiClientCallback) client_cb,
-						  browser,
-						  &error);
-#endif
-#ifdef HAVE_AVAHI_0_6
-	{
-		AvahiClientFlags flags;
-		flags = 0;
-
-		browser->priv->client = avahi_client_new (avahi_glib_poll_get (browser->priv->poll),
-							  flags,
-							  (AvahiClientCallback)client_cb,
-							  browser,
-							  &error);
-	}
-#endif
-}
-
 static void
 resolve_cb (AvahiServiceResolver  *service_resolver,
 	    AvahiIfIndex           interface,
@@ -160,9 +102,7 @@ resolve_cb (AvahiServiceResolver  *service_resolver,
 	    const AvahiAddress    *address,
 	    uint16_t               port,
 	    AvahiStringList       *text,
-#ifdef HAVE_AVAHI_0_6
 	    AvahiLookupResultFlags flags,
-#endif
 	    RBDaapMdnsBrowser     *browser)
 {
 	if (event == AVAHI_RESOLVER_FOUND) {
@@ -232,9 +172,7 @@ rb_daap_mdns_browser_resolve (RBDaapMdnsBrowser *browser,
 						       "_daap._tcp",
 						       NULL,
 						       AVAHI_PROTO_UNSPEC,
-#ifdef HAVE_AVAHI_0_6
 						       0,
-#endif
 						       (AvahiServiceResolverCallback)resolve_cb,
 						       browser);
 	if (service_resolver == NULL) {
@@ -269,19 +207,12 @@ browse_cb (AvahiServiceBrowser   *service_browser,
 	   const char            *name,
 	   const char            *type,
 	   const char            *domain,
-#ifdef HAVE_AVAHI_0_6
 	   AvahiLookupResultFlags flags,
-#endif
 	   RBDaapMdnsBrowser     *browser)
 {
 	gboolean local;
 
-#ifdef HAVE_AVAHI_0_5
-	local = avahi_client_is_service_local (browser->priv->client, interface, protocol, name, type, domain);
-#endif
-#ifdef HAVE_AVAHI_0_6
-	local = ((flags & AVAHI_LOOKUP_RESULT_LOCAL) != 0);
-#endif
+	local = ((flags & AVAHI_LOOKUP_RESULT_OUR_OWN) != 0);
 	if (local) {
 		rb_debug ("Ignoring local service %s", name);
 		return;
@@ -320,9 +251,7 @@ rb_daap_mdns_browser_start (RBDaapMdnsBrowser *browser,
 								    AVAHI_PROTO_UNSPEC,
 								    "_daap._tcp",
 								    NULL,
-#ifdef HAVE_AVAHI_0_6
 								    0,
-#endif
 								    (AvahiServiceBrowserCallback)browse_cb,
 								    browser);
 	if (browser->priv->service_browser == NULL) {
@@ -432,7 +361,7 @@ rb_daap_mdns_browser_init (RBDaapMdnsBrowser *browser)
 {
 	browser->priv = RB_DAAP_MDNS_BROWSER_GET_PRIVATE (browser);
 
-	avahi_client_init (browser);
+	browser->priv->client = rb_daap_mdns_avahi_get_client ();
 }
 
 static void
@@ -458,14 +387,6 @@ rb_daap_mdns_browser_finalize (GObject *object)
 		avahi_service_browser_free (browser->priv->service_browser);
 	}
 
-	if (browser->priv->client) {
-		avahi_client_free (browser->priv->client);
-	}
-
-	if (browser->priv->poll) {
-		avahi_glib_poll_free (browser->priv->poll);
-	}
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
