@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -66,6 +74,7 @@ static gboolean rb_daap_source_show_popup (RBSource *source);
 static char * rb_daap_source_get_browser_key (RBSource *source);
 static char * rb_daap_source_get_paned_key (RBBrowserSource *source);
 static void rb_daap_source_get_status (RBSource *source, char **text, char **progress_text, float *progress);
+static char * rb_daap_source_get_playback_uri (RhythmDBEntry *entry, gpointer data);
 
 #define CONF_STATE_SORTING CONF_PREFIX "/state/daap/sorting"
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/daap/paned_position"
@@ -136,7 +145,6 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	object_class->set_property = rb_daap_source_set_property;
 
 	source_class->impl_activate = rb_daap_source_activate;
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
@@ -268,11 +276,15 @@ rb_daap_source_new (RBShell *shell,
 	RhythmDBEntryType type;
 	GdkPixbuf *icon;
 	RhythmDB *db;
+	char *entry_type_name;
 
 	g_object_get (shell, "db", &db, NULL);
-	type = rhythmdb_entry_register_type (db, NULL);
+	entry_type_name = g_strdup_printf ("daap:%s:%s:%s", service_name, name, host);
+	type = rhythmdb_entry_register_type (db, entry_type_name);
+	g_free (entry_type_name);
 	type->save_to_disk = FALSE;
 	type->category = RHYTHMDB_ENTRY_NORMAL;
+	type->get_playback_uri = (RhythmDBEntryStringFunc) rb_daap_source_get_playback_uri;
 	g_object_unref (db);
 
 	icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin), password_protected, FALSE);
@@ -306,7 +318,7 @@ connection_auth_cb (RBDAAPConnection *connection,
 		    const char       *name,
 		    RBDAAPSource     *source)
 {
-	gchar *password;
+	gchar *password = NULL;
 #ifdef WITH_GNOME_KEYRING
 	GnomeKeyringResult keyringret;
 	gchar *keyring;
@@ -327,10 +339,14 @@ connection_auth_cb (RBDAAPConnection *connection,
 	if (keyringret == GNOME_KEYRING_RESULT_OK) {
 		GnomeKeyringNetworkPasswordData *pwd_data;
 
-		pwd_data = (GnomeKeyringNetworkPasswordData*)list->data;
-		password = g_strdup (pwd_data->password);
+		if (list != NULL) {
+			pwd_data = (GnomeKeyringNetworkPasswordData*)list->data;
+			password = g_strdup (pwd_data->password);
+		}
 		source->priv->tried_password = TRUE;
-	} else {
+	}
+
+	if (password == NULL) {
 #endif
 		GtkWindow *parent;
 		GnomePasswordDialog *dialog;
@@ -525,8 +541,14 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 	for (l = playlists; l != NULL; l = g_slist_next (l)) {
 		RBDAAPPlaylist *playlist = l->data;
 		RBSource *playlist_source;
+		char *sorting_name;
+
+		/* Construct a unique sorting name for this playlist, as <Share Name>_<Playlist> */
+		sorting_name = g_strjoin (NULL, daap_source->priv->service_name, "_", playlist->name, NULL);
+
+		playlist_source = rb_static_playlist_source_new (shell, playlist->name, sorting_name, FALSE, entry_type);
+		g_free (sorting_name);
 
-		playlist_source = rb_static_playlist_source_new (shell, playlist->name, FALSE, entry_type);
 		g_list_foreach (playlist->uris, (GFunc)_add_location_to_playlist, playlist_source);
 
 		rb_shell_append_source (shell, playlist_source, RB_SOURCE (daap_source));
@@ -563,6 +585,7 @@ rb_daap_source_activate (RBSource *source)
 								daap_source->priv->password_protected,
 								db,
 								type);
+        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, type);
 	g_object_add_weak_pointer (G_OBJECT (daap_source->priv->connection), (gpointer *)&daap_source->priv->connection);
 
 	g_free (name);
@@ -611,7 +634,8 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 	RhythmDB *db;
 	RhythmDBEntryType type;
 
-	if (daap_source->priv->connection == NULL) {
+	if (daap_source->priv->connection == NULL
+	 || daap_source->priv->disconnecting == TRUE) {
 		return;
 	}
 
@@ -624,6 +648,7 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 	g_object_unref (shell);
 
 	rhythmdb_entry_delete_by_type (db, type);
+        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, type);
 	rhythmdb_commit (db);
 
 	g_object_unref (db);
@@ -662,6 +687,8 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 		rb_debug ("Waiting for DAAP connection to finish...");
 		gtk_main_iteration ();
 	}
+
+	daap_source->priv->disconnecting = FALSE;
 	rb_debug ("DAAP connection finished");
 }
 
@@ -729,3 +756,17 @@ rb_daap_source_get_status (RBSource *source,
 
 	RB_SOURCE_CLASS (rb_daap_source_parent_class)->impl_get_status (source, text, progress_text, progress);
 }
+
+static char *
+rb_daap_source_get_playback_uri (RhythmDBEntry *entry, gpointer data)
+{
+	const char *location;
+
+	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
+	if (location == NULL) {
+		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	}
+
+	return g_strdup (location);
+}
+
