@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -36,8 +36,6 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 
 #ifdef HAVE_HAL
 #include <libhal.h>
@@ -54,13 +52,10 @@
 #include "rb-plugin.h"
 
 static void rb_psp_source_create_playlists (RBGenericPlayerSource *source);
-static gchar *impl_get_mount_path (RBGenericPlayerSource *source);
 
 typedef struct
 {
-#ifdef __SUNPRO_C
-  char x; /* To build with Solaris forte compiler. */
-#endif
+  char garbage_so_its_not_empty; /* To avoid run-time warnings. FIXME remove if no private needed */
 } RBPspSourcePrivate;
 
 
@@ -73,7 +68,6 @@ rb_psp_source_class_init (RBPspSourceClass *klass)
 {
 	RBGenericPlayerSourceClass *generic_class = RB_GENERIC_PLAYER_SOURCE_CLASS (klass);
 
-	generic_class->impl_get_mount_path = impl_get_mount_path;
 	generic_class->impl_load_playlists = rb_psp_source_create_playlists;
 
 	g_type_class_add_private (klass, sizeof (RBPspSourcePrivate));
@@ -86,27 +80,31 @@ rb_psp_source_init (RBPspSource *source)
 }
 
 RBRemovableMediaSource *
-rb_psp_source_new (RBShell *shell, GnomeVFSVolume *volume)
+rb_psp_source_new (RBShell *shell, GMount *mount)
 {
 	RBPspSource *source;
 	RhythmDBEntryType entry_type;
 	RhythmDB *db;
 	char *name;
 	char *path;
+	GVolume *volume;
 
-	g_assert (rb_psp_is_volume_player (volume));
+	g_assert (rb_psp_is_mount_player (mount));
+
+	volume = g_mount_get_volume (mount);
 
 	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-	path = gnome_vfs_volume_get_device_path (volume);
+	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	name = g_strdup_printf ("psp: %s", path);
 	entry_type = rhythmdb_entry_register_type (db, name);
-	g_object_unref (G_OBJECT (db));
+	g_object_unref (db);
 	g_free (name);
 	g_free (path);
+	g_object_unref (volume);
 
 	source = RB_PSP_SOURCE (g_object_new (RB_TYPE_PSP_SOURCE,
 					  "entry-type", entry_type,
-					  "volume", volume,
+					  "mount", mount,
 					  "shell", shell,
 					  "source-group", RB_SOURCE_GROUP_DEVICES,
 					  NULL));
@@ -116,55 +114,94 @@ rb_psp_source_new (RBShell *shell, GnomeVFSVolume *volume)
 	return RB_REMOVABLE_MEDIA_SOURCE (source);
 }
 
-static char *
-impl_get_mount_path (RBGenericPlayerSource *source)
+static GFile *
+find_dir_no_case (GFile *root, gboolean look_for_psp)
 {
-	char *uri;
-	char *path;
-	GnomeVFSVolume *volume;
+	GFileEnumerator *e;
+	GFileInfo *info;
+	GFile *ret;
+	GFile *music_dir;
 
-	g_object_get (G_OBJECT (source), "volume", &volume, NULL);
-	uri = gnome_vfs_volume_get_activation_uri (volume);
-	g_object_unref (volume);
+	ret = music_dir = NULL;
+	e = g_file_enumerate_children (root, G_FILE_ATTRIBUTE_STANDARD_NAME","G_FILE_ATTRIBUTE_STANDARD_TYPE,
+				       G_FILE_QUERY_INFO_NONE, NULL, NULL);
+	if (e == NULL)
+		return ret;
 
-	path = rb_uri_append_path (uri, "PSP/MUSIC");
-	/* For Firmware versions 2.80+ */
-	if (!rb_uri_exists(path)) {
-		g_free (path);
-		path = rb_uri_append_path(uri, "MUSIC");
+	while ((info = g_file_enumerator_next_file (e, NULL, NULL)) != NULL) {
+		const char *name;
+
+		name = g_file_info_get_name (info);
+		if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY) {
+			g_object_unref (info);
+			continue;
+		}
+
+		if (g_ascii_strcasecmp (name, "MUSIC") == 0) {
+			music_dir = g_file_resolve_relative_path (root, name);
+			g_object_unref (info);
+			if (look_for_psp)
+				continue;
+			else
+				break;
+		}
+
+		if (look_for_psp) {
+			if (g_ascii_strcasecmp (name, "PSP") == 0) {
+				GFile *psp_dir;
+
+				psp_dir = g_file_resolve_relative_path (root, name);
+				ret = find_dir_no_case (psp_dir, FALSE);
+				g_object_unref (psp_dir);
+
+				if (ret != NULL) {
+					g_object_unref (info);
+					if (music_dir != NULL)
+						g_object_unref (music_dir);
+					music_dir = NULL;
+					break;
+				}
+			}
+		}
+		g_object_unref (info);
 	}
+	g_object_unref (e);
 
-	g_free (uri);
+	if (ret == NULL)
+		ret = music_dir;
 
-	return path;
+	return ret;
 }
 
-static gboolean
-visit_playlist_dirs (const gchar *rel_path,
-		     GnomeVFSFileInfo *info,
-		     gboolean recursing_will_loop,
-		     RBPspSource *source,
-		     gboolean *recurse)
+static GFile *
+find_music_dir (GMount *mount)
+{
+	GFile *root;
+	GFile *music_dir = NULL;
+
+	root = g_mount_get_root (mount);
+	if (root != NULL) {
+		music_dir = find_dir_no_case (root, TRUE);
+		/* FIXME create directories if they don't exist */
+		g_object_unref (root);
+	}
+
+	return music_dir;
+}
+
+static void
+visit_playlist_dirs (RBPspSource *source,
+		     GFile *file)
 {
 	RBShell *shell;
 	RhythmDB *db;
 	RhythmDBEntryType entry_type;
-	char *main_path;
 	char *playlist_path;
+	char *playlist_name;
 	RBSource *playlist;
 	GPtrArray *query;
 
-	*recurse = FALSE;
-
-	/* add playlist */
-	main_path = rb_generic_player_source_get_mount_path (RB_GENERIC_PLAYER_SOURCE (source));
-	playlist_path = rb_uri_append_path (main_path, rel_path);
-	g_free (main_path);
-
-	if (!rb_uri_is_directory (playlist_path)) {
-		g_free (playlist_path);
-		return TRUE;
-	}
+	playlist_path = g_file_get_uri (file);		/* or _get_path? */
 
 	g_object_get (source,
 		      "shell", &shell,
@@ -174,6 +211,8 @@ visit_playlist_dirs (const gchar *rel_path,
 		      "db", &db,
 		      NULL);
 
+	/* FIXME this isn't good enough, we only need the files directly under the playlist directory,
+	 * not sub-dirs */
 	query = rhythmdb_query_parse (db,
 				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, entry_type,
 				      RHYTHMDB_QUERY_PROP_PREFIX, RHYTHMDB_PROP_LOCATION, playlist_path,
@@ -181,7 +220,10 @@ visit_playlist_dirs (const gchar *rel_path,
 	g_free (playlist_path);
         g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
-	playlist = rb_auto_playlist_source_new (shell, rel_path, FALSE);
+	playlist_name = g_file_get_basename (file);
+	playlist = rb_auto_playlist_source_new (shell, playlist_name, FALSE);
+	g_free (playlist_name);
+
 	rb_auto_playlist_source_set_query (RB_AUTO_PLAYLIST_SOURCE (playlist), query,
 					   RHYTHMDB_QUERY_MODEL_LIMIT_NONE, NULL,
 					   NULL, 0);
@@ -190,23 +232,43 @@ visit_playlist_dirs (const gchar *rel_path,
 
 	g_object_unref (shell);
 	g_object_unref (db);
-
-	return TRUE;
 }
 
 
 static void
 rb_psp_source_create_playlists (RBGenericPlayerSource *source)
 {
-	char *mount_path;
+	GMount *mount;
+	GFile *music_dir;
 
-	mount_path = rb_generic_player_source_get_mount_path (source);
-	gnome_vfs_directory_visit (mount_path,
-				   GNOME_VFS_FILE_INFO_DEFAULT,
-				   GNOME_VFS_DIRECTORY_VISIT_DEFAULT,
-				   (GnomeVFSDirectoryVisitFunc) visit_playlist_dirs,
-				   source);
-	g_free (mount_path);
+	g_object_get (source, "mount", &mount, NULL);
+	music_dir = find_music_dir (mount);
+	g_object_unref (mount);
+
+	if (music_dir != NULL) {
+		GFileEnumerator *e;
+		GFileInfo *info;
+
+		e = g_file_enumerate_children (music_dir, G_FILE_ATTRIBUTE_STANDARD_NAME","G_FILE_ATTRIBUTE_STANDARD_TYPE,
+					       G_FILE_QUERY_INFO_NONE, NULL, NULL);
+		if (e != NULL) {
+			while ((info = g_file_enumerator_next_file (e, NULL, NULL)) != NULL) {
+				GFile *file;
+				const char *name;
+				if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY) {
+					g_object_unref (info);
+					continue;
+				}
+				name = g_file_info_get_name (info);
+				file = g_file_resolve_relative_path (music_dir, name);
+				visit_playlist_dirs (RB_PSP_SOURCE (source), file);
+				g_object_unref (file);
+				g_object_unref (info);
+			}
+			g_object_unref (e);
+		}
+		g_object_unref (music_dir);
+	}
 }
 
 #ifdef HAVE_HAL
@@ -253,7 +315,7 @@ hal_udi_is_psp (const char *udi)
 	if (parent_name == NULL || dbus_error_is_set (&error))
 		goto end;
 
-	if (strcmp (parent_name, "PSP") == 0) {
+	if (strcmp (parent_name, "PSP") == 0 || strcmp (parent_name, "\"PSP\" MS") == 0) {
 		result = TRUE;
 	}
 
@@ -280,39 +342,32 @@ end:
 #endif
 
 gboolean
-rb_psp_is_volume_player (GnomeVFSVolume *volume)
+rb_psp_is_mount_player (GMount *mount)
 {
+#ifndef HAVE_HAL
+	GFile *music_dir;
+#else
+	GVolume *volume;
+#endif
 	gboolean result = FALSE;
-	gchar *str;
-
-	if (gnome_vfs_volume_get_volume_type (volume) != GNOME_VFS_VOLUME_TYPE_MOUNTPOINT) {
-		return FALSE;
-	}
 
 #ifndef HAVE_HAL
-	str = gnome_vfs_volume_get_activation_uri (volume);
-	if (str) {
-		char *path;
-
-		path = rb_uri_append_path (str, "PSP/MUSIC");
-		g_free (str);
-		result = rb_uri_exists (path);
-		if (!result) {
-			g_free (path);
-			path = rb_uri_append_path (str, "MUSIC");
-			result = rb_uri_exists (path);
-		}
-		g_free (path);
-		return result;
+	music_dir = find_music_dir (mount);
+	if (music_dir != NULL) {
+		g_object_unref (music_dir);
+		result = TRUE;
 	}
 #else
-	str = gnome_vfs_volume_get_hal_udi (volume);
-	if (str != NULL) {
-		gboolean result;
+	volume = g_mount_get_volume (mount);
+	if (volume != NULL) {
+		char *str;
 
-		result = hal_udi_is_psp (str);
-		g_free (str);
-		return result;
+		str = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+		if (str != NULL) {
+			result = hal_udi_is_psp (str);
+			g_free (str);
+		}
+		g_object_unref (volume);
 	}
 #endif
 	return result;
