@@ -8,6 +8,14 @@
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  *
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -28,7 +36,6 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 #include <glib-object.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <libmtp.h>
 #include <hal/libhal.h>
 #include <dbus/dbus.h>
@@ -83,12 +90,9 @@ static void impl_deactivate (RBPlugin *plugin, RBShell *shell);
 
 static void rb_mtp_plugin_device_added (LibHalContext *context, const char *udi);
 static void rb_mtp_plugin_device_removed (LibHalContext *context, const char *udi);
-static void rb_mtp_plugin_setup_dbus_hal_connection (RBMtpPlugin *plugin);
+static gboolean rb_mtp_plugin_setup_dbus_hal_connection (RBMtpPlugin *plugin);
 
 static RBSource* create_source_cb (RBMtpPlugin *plugin, LIBMTP_mtpdevice_t *device, const char *udi);
-/*static RBSource * create_source_cb (RBRemovableMediaManager *rmm,
-				    GnomeVFSVolume *volume,
-				    RBMtpPlugin *plugin);*/
 static void rb_mtp_plugin_eject  (GtkAction *action, RBMtpPlugin *plugin);
 static void rb_mtp_plugin_rename (GtkAction *action, RBMtpPlugin *plugin);
 
@@ -161,10 +165,14 @@ impl_activate (RBPlugin *bplugin, RBShell *shell)
 	gtk_ui_manager_insert_action_group (uimanager, plugin->action_group, 0);
 	file = rb_plugin_find_file (bplugin, "mtp-ui.xml");
 	plugin->ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager, file, NULL);
+	g_object_unref (G_OBJECT (uimanager));
 
-	/*device detection*/
+	/* device detection */
 
-	rb_mtp_plugin_setup_dbus_hal_connection (plugin);
+	if (rb_mtp_plugin_setup_dbus_hal_connection (plugin) == FALSE) {
+		rb_debug ("not scanning for MTP devices because we couldn't get a HAL context");
+		return;
+	}
 
 	rb_profile_start ("scanning for MTP devices");
 	devices = libhal_get_all_devices (plugin->hal_context, &num, NULL);
@@ -196,10 +204,9 @@ impl_activate (RBPlugin *bplugin, RBShell *shell)
 	} else {
 		rb_debug ("Couldn't list mtp devices");
 	}
+
 	libhal_free_string_array (devices);
 	rb_profile_end ("scanning for MTP devices");
-
-	g_object_unref (G_OBJECT (uimanager));
 }
 
 static void
@@ -207,7 +214,6 @@ impl_deactivate (RBPlugin *bplugin, RBShell *shell)
 {
 	RBMtpPlugin *plugin = RB_MTP_PLUGIN (bplugin);
 	GtkUIManager *uimanager = NULL;
-	DBusError error;
 
 	g_object_get (G_OBJECT (shell),
 		      "ui-manager", &uimanager,
@@ -220,11 +226,20 @@ impl_deactivate (RBPlugin *bplugin, RBShell *shell)
 	g_list_free (plugin->mtp_sources);
 	plugin->mtp_sources = NULL;
 
-	dbus_error_init (&error);
-	libhal_ctx_shutdown (plugin->hal_context, &error);
-	libhal_ctx_free (plugin->hal_context);
-	dbus_connection_unref (plugin->dbus_connection);
-	dbus_error_free (&error);
+	if (plugin->hal_context != NULL) {
+		DBusError error;
+		dbus_error_init (&error);
+		libhal_ctx_shutdown (plugin->hal_context, &error);
+		libhal_ctx_free (plugin->hal_context);
+		dbus_error_free (&error);
+
+		plugin->hal_context = NULL;
+	}
+
+	if (plugin->dbus_connection != NULL) {
+		dbus_connection_unref (plugin->dbus_connection);
+		plugin->dbus_connection = NULL;
+	}
 
 	g_object_unref (G_OBJECT (uimanager));
 }
@@ -263,7 +278,7 @@ rb_mtp_plugin_eject (GtkAction *action,
 		      "selected-source", &source,
 		      NULL);
 	if ((source == NULL) || !RB_IS_MTP_SOURCE (source)) {
-		g_critical ("got MTPSourceEject action for non-mtp source");
+		g_warning ("got MTPSourceEject action for non-mtp source");
 		if (source != NULL)
 			g_object_unref (source);
 		return;
@@ -288,7 +303,7 @@ rb_mtp_plugin_rename (GtkAction *action,
 		      "selected-source", &source,
 		      NULL);
 	if ((source == NULL) || !RB_IS_MTP_SOURCE (source)) {
-		g_critical ("got MTPSourceEject action for non-mtp source");
+		g_warning ("got MTPSourceEject action for non-mtp source");
 		if (source != NULL)
 			g_object_unref (source);
 		return;
@@ -356,16 +371,19 @@ rb_mtp_plugin_device_removed (LibHalContext *context, const char *udi)
 
 	for (tmp = list; tmp != NULL; tmp = tmp->next) {
 		RBSource *source = (RBSource *)tmp->data;
-		if ((source != NULL) && (rb_mtp_source_is_udi (RB_MTP_SOURCE (source), udi) != 0)) {
+		char *source_udi;
+
+		g_object_get (source, "udi", &source_udi, NULL);
+		if (strcmp (udi, source_udi) == 0) {
 			rb_debug ("removing device %s, %p", udi, source);
 			plugin->mtp_sources = g_list_remove (plugin->mtp_sources, source);
 			rb_source_delete_thyself (source);
-			break;
 		}
+		g_free (source_udi);
 	}
 }
 
-static void
+static gboolean
 rb_mtp_plugin_setup_dbus_hal_connection (RBMtpPlugin *plugin)
 {
 	DBusError error;
@@ -374,7 +392,8 @@ rb_mtp_plugin_setup_dbus_hal_connection (RBMtpPlugin *plugin)
 	plugin->dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
 	if (plugin->dbus_connection == NULL) {
 		rb_debug ("error: dbus_bus_get: %s: %s\n", error.name, error.message);
-		return;
+		dbus_error_free (&error);
+		return FALSE;
 	}
 
 	dbus_connection_setup_with_g_main (plugin->dbus_connection, NULL);
@@ -383,7 +402,8 @@ rb_mtp_plugin_setup_dbus_hal_connection (RBMtpPlugin *plugin)
 
 	plugin->hal_context = libhal_ctx_new ();
 	if (plugin->hal_context == NULL) {
-		return;
+		dbus_error_free (&error);
+		return FALSE;
 	}
 	libhal_ctx_set_dbus_connection (plugin->hal_context, plugin->dbus_connection);
 
@@ -395,9 +415,10 @@ rb_mtp_plugin_setup_dbus_hal_connection (RBMtpPlugin *plugin)
 	if (!libhal_ctx_init (plugin->hal_context, &error)) {
 		rb_debug ("error: libhal_ctx_init: %s: %s\n", error.name, error.message);
 		dbus_error_free (&error);
-		return;
+		return FALSE;
 	}
 
 	dbus_error_free (&error);
+	return TRUE;
 }
 
