@@ -8,7 +8,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -30,11 +30,8 @@
 #include <config.h>
 
 #include <string.h>
-#include <gtk/gtktreeview.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
@@ -47,17 +44,61 @@
 #include "rb-refstring.h"
 #include "rhythmdb.h"
 #include "rb-encoder.h"
+#include "rb-dialog.h"
 
 #include "rb-mtp-source.h"
 
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/mtp/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/mtp/show_browser"
 
+#ifndef HAVE_LIBMTP_030
+/*
+ * Helpful macros to determine filetype properties
+ */
+#define LIBMTP_FILETYPE_IS_AUDIO(a)\
+(a == LIBMTP_FILETYPE_WAV ||\
+ a == LIBMTP_FILETYPE_MP3 ||\
+ a == LIBMTP_FILETYPE_MP2 ||\
+ a == LIBMTP_FILETYPE_WMA ||\
+ a == LIBMTP_FILETYPE_OGG ||\
+ a == LIBMTP_FILETYPE_FLAC ||\
+ a == LIBMTP_FILETYPE_AAC ||\
+ a == LIBMTP_FILETYPE_M4A ||\
+ a == LIBMTP_FILETYPE_UNDEF_AUDIO)
+
+#define LIBMTP_FILETYPE_IS_VIDEO(a)\
+(a == LIBMTP_FILETYPE_WMV ||\
+ a == LIBMTP_FILETYPE_AVI ||\
+ a == LIBMTP_FILETYPE_MPEG ||\
+ a == LIBMTP_FILETYPE_UNDEF_VIDEO)
+
+#define LIBMTP_FILETYPE_IS_AUDIOVIDEO(a)\
+(a == LIBMTP_FILETYPE_MP4 ||\
+ a == LIBMTP_FILETYPE_ASF ||\
+ a == LIBMTP_FILETYPE_QT)
+
+#define LIBMTP_FILETYPE_IS_TRACK(a)\
+(LIBMTP_FILETYPE_IS_AUDIO(a) ||\
+ LIBMTP_FILETYPE_IS_VIDEO(a) ||\
+ LIBMTP_FILETYPE_IS_AUDIOVIDEO(a))
+
+#endif
+
+
 static GObject *rb_mtp_source_constructor (GType type,
 					   guint n_construct_properties,
 					   GObjectConstructParam *construct_properties);
+static void rb_mtp_source_dispose (GObject *object);
 static void rb_mtp_source_finalize (GObject *object);
 
+static void rb_mtp_source_set_property (GObject *object,
+			                guint prop_id,
+			                const GValue *value,
+			                GParamSpec *pspec);
+static void rb_mtp_source_get_property (GObject *object,
+			                guint prop_id,
+			                GValue *value,
+			                GParamSpec *pspec);
 static char *impl_get_browser_key (RBSource *source);
 static char *impl_get_paned_key (RBBrowserSource *source);
 
@@ -68,7 +109,6 @@ static gboolean rb_mtp_source_transfer_track_to_disk (LIBMTP_mtpdevice_t *device
 static char* rb_mtp_source_get_playback_uri (RhythmDBEntry *entry,
 					     gpointer data);
 
-static void impl_delete_thyself (RBSource *source);
 static void impl_delete (RBSource *asource);
 static gboolean impl_show_popup (RBSource *source);
 static GList* impl_get_ui_actions (RBSource *source);
@@ -92,6 +132,10 @@ typedef struct
 	LIBMTP_mtpdevice_t *device;
 	GHashTable *entry_map;
 	char *udi;
+	uint16_t supported_types[LIBMTP_FILETYPE_UNKNOWN+1];
+	GList *mediatypes;
+
+	guint load_songs_idle_id;
 } RBMtpSourcePrivate;
 
 RB_PLUGIN_DEFINE_TYPE(RBMtpSource,
@@ -100,6 +144,32 @@ RB_PLUGIN_DEFINE_TYPE(RBMtpSource,
 
 #define MTP_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_MTP_SOURCE, RBMtpSourcePrivate))
 
+enum
+{
+	PROP_0,
+	PROP_LIBMTP_DEVICE,
+	PROP_UDI,
+};
+
+static void
+report_libmtp_errors (LIBMTP_mtpdevice_t *device, gboolean use_dialog)
+{
+	LIBMTP_error_t *stack;
+
+	for (stack = LIBMTP_Get_Errorstack (device); stack != NULL; stack = stack->next) {
+		if (use_dialog) {
+			rb_error_dialog (NULL, _("Media player device error"), "%s", stack->error_text);
+
+			/* only display one dialog box per error */
+			use_dialog = FALSE;
+		} else {
+			g_warning ("libmtp error: %s", stack->error_text);
+		}
+	}
+
+	LIBMTP_Clear_Errorstack (device);
+}
+
 static void
 rb_mtp_source_class_init (RBMtpSourceClass *klass)
 {
@@ -109,23 +179,23 @@ rb_mtp_source_class_init (RBMtpSourceClass *klass)
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
 	object_class->constructor = rb_mtp_source_constructor;
+	object_class->dispose = rb_mtp_source_dispose;
 	object_class->finalize = rb_mtp_source_finalize;
+	object_class->set_property = rb_mtp_source_set_property;
+	object_class->get_property = rb_mtp_source_get_property;
 
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_browser_key = impl_get_browser_key;
 
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
-#ifdef ENABLE_TRACK_TRANSFER
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
-#endif
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
-	source_class->impl_delete_thyself = impl_delete_thyself;
 	source_class->impl_delete = impl_delete;
 	source_class->impl_copy = impl_copy;
 
@@ -134,6 +204,21 @@ rb_mtp_source_class_init (RBMtpSourceClass *klass)
 	rms_class->impl_track_added = impl_track_added;
 	rms_class->impl_build_dest_uri = impl_build_dest_uri;
 	rms_class->impl_get_mime_types = impl_get_mime_types;
+	rms_class->impl_should_paste = rb_removable_media_source_should_paste_no_duplicate;
+
+	g_object_class_install_property (object_class,
+					 PROP_LIBMTP_DEVICE,
+					 g_param_spec_pointer ("libmtp-device",
+							       "libmtp-device",
+							       "libmtp device",
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_UDI,
+					 g_param_spec_string ("udi",
+						 	      "udi",
+							      "udi",
+							      NULL,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (klass, sizeof (RBMtpSourcePrivate));
 }
@@ -144,16 +229,13 @@ rb_mtp_source_name_changed_cb (RBMtpSource *source,
 			       gpointer data)
 {
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	char *name = NULL;
 
-	if (priv->device) {
-		char *name = NULL;
-
-		g_object_get (source, "name", &name, NULL);
-		if (LIBMTP_Set_Friendlyname (priv->device, name) != 0) {
-			rb_debug ("Set friendly name failed");
-		}
-		g_free (name);
+	g_object_get (source, "name", &name, NULL);
+	if (LIBMTP_Set_Friendlyname (priv->device, name) != 0) {
+		report_libmtp_errors (priv->device, TRUE);
 	}
+	g_free (name);
 }
 
 static void
@@ -161,11 +243,10 @@ rb_mtp_source_init (RBMtpSource *source)
 {
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
 
-	priv->entry_map = g_hash_table_new (g_direct_hash,
-					    g_direct_equal);
-
-	g_signal_connect (G_OBJECT (source), "notify::name",
-			  (GCallback)rb_mtp_source_name_changed_cb, NULL);
+	priv->entry_map = g_hash_table_new_full (g_direct_hash,
+						 g_direct_equal,
+						 NULL,
+						 (GDestroyNotify) LIBMTP_destroy_track_t);
 }
 
 static GObject *
@@ -173,10 +254,13 @@ rb_mtp_source_constructor (GType type, guint n_construct_properties,
 			   GObjectConstructParam *construct_properties)
 {
 	RBMtpSource *source;
+	RBMtpSourcePrivate *priv;
 	RBEntryView *tracks;
 	GtkIconTheme *theme;
 	GdkPixbuf *pixbuf;
 	gint size;
+	guint16 *types = NULL;
+	guint16 num_types= 0;
 
 	source = RB_MTP_SOURCE (G_OBJECT_CLASS (rb_mtp_source_parent_class)->
 				constructor (type, n_construct_properties, construct_properties));
@@ -193,12 +277,159 @@ rb_mtp_source_constructor (GType type, guint n_construct_properties,
 	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
 	g_object_unref (pixbuf);
 
+	g_signal_connect (G_OBJECT (source), "notify::name",
+			  (GCallback)rb_mtp_source_name_changed_cb, NULL);
+	
+	/* figure out supported file types */
+	priv = MTP_SOURCE_GET_PRIVATE (source);
+	if (LIBMTP_Get_Supported_Filetypes(priv->device, &types, &num_types) == 0) {
+		int i;
+		gboolean has_mp3 = FALSE;
+		for (i = 0; i < num_types; i++) {
+			const char *mediatype;
+
+			if (i <= LIBMTP_FILETYPE_UNKNOWN) {
+				priv->supported_types[types[i]] = 1;
+			}
+
+			/* this has to work with the remapping done in 
+			 * rb-removable-media-source.c:impl_paste.
+			 */
+			switch (types[i]) {
+			case LIBMTP_FILETYPE_WAV:
+				mediatype = "audio/x-wav";
+				break;
+			case LIBMTP_FILETYPE_MP3:
+				/* special handling for mp3: always put it at the front of the list
+				 * if it's supported.
+				 */
+				has_mp3 = TRUE;
+				mediatype = NULL;
+				break;
+			case LIBMTP_FILETYPE_WMA:
+				mediatype = "audio/x-ms-wma";
+				break;
+			case LIBMTP_FILETYPE_OGG:
+				mediatype = "application/ogg";
+				break;
+			case LIBMTP_FILETYPE_MP4:
+			case LIBMTP_FILETYPE_M4A:
+			case LIBMTP_FILETYPE_AAC:
+				mediatype = "audio/aac";
+				break;
+			case LIBMTP_FILETYPE_WMV:
+				mediatype = "audio/x-ms-wmv";
+				break;
+			case LIBMTP_FILETYPE_ASF:
+				mediatype = "video/x-ms-asf";
+				break;
+			case LIBMTP_FILETYPE_FLAC:
+				mediatype = "audio/flac";
+				break;
+			default:
+				rb_debug ("unknown libmtp filetype %s supported", LIBMTP_Get_Filetype_Description (types[i]));
+				mediatype = NULL;
+				break;
+			}
+
+			if (mediatype != NULL) {
+				rb_debug ("media type %s supported", mediatype);
+				priv->mediatypes = g_list_prepend (priv->mediatypes,
+								   g_strdup (mediatype));
+			}
+		}
+
+		if (has_mp3) {
+			rb_debug ("audio/mpeg supported");
+			priv->mediatypes = g_list_prepend (priv->mediatypes, g_strdup ("audio/mpeg"));
+		}
+	} else {
+		report_libmtp_errors (priv->device, FALSE);
+	}
+
+	rb_mtp_source_load_tracks (source);
+
 	return G_OBJECT (source);
 }
 
+static void
+rb_mtp_source_set_property (GObject *object,
+			    guint prop_id,
+			    const GValue *value,
+			    GParamSpec *pspec)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_LIBMTP_DEVICE:
+		priv->device = g_value_get_pointer (value);
+		break;
+	case PROP_UDI:
+		priv->udi = g_value_dup_string (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+rb_mtp_source_get_property (GObject *object,
+			    guint prop_id,
+			    GValue *value,
+			    GParamSpec *pspec)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_LIBMTP_DEVICE:
+		g_value_set_pointer (value, priv->device);
+		break;
+	case PROP_UDI:
+		g_value_set_string (value, priv->udi);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+rb_mtp_source_dispose (GObject *object)
+{
+	RBMtpSource *source = RB_MTP_SOURCE (object);
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	RhythmDBEntryType entry_type;
+	RhythmDB *db;
+
+	db = get_db_for_source (source);
+
+	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
+	rhythmdb_entry_delete_by_type (db, entry_type);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+
+	rhythmdb_commit (db);
+	g_object_unref (db);
+
+	if (priv->load_songs_idle_id != 0) {
+		g_source_remove (priv->load_songs_idle_id);
+		priv->load_songs_idle_id = 0;
+	}
+
+	G_OBJECT_CLASS (rb_mtp_source_parent_class)->dispose (object);
+}
+
 static void
 rb_mtp_source_finalize (GObject *object)
 {
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (object);
+
+	g_hash_table_destroy (priv->entry_map);
+
+	g_free (priv->udi);
+
+	LIBMTP_Release_Device (priv->device);
+
 	G_OBJECT_CLASS (rb_mtp_source_parent_class)->finalize (object);
 }
 
@@ -222,7 +453,6 @@ rb_mtp_source_new (RBShell *shell,
 	RBMtpSource *source = NULL;
 	RhythmDBEntryType entry_type;
 	RhythmDB *db = NULL;
-	RBMtpSourcePrivate *priv = NULL;
 	char *name = NULL;
 
 	g_object_get (shell, "db", &db, NULL);
@@ -242,16 +472,12 @@ rb_mtp_source_new (RBShell *shell,
 					      "visibility", TRUE,
 					      "volume", NULL,
 					      "source-group", RB_SOURCE_GROUP_DEVICES,
+					      "libmtp-device", device,
+					      "udi", udi,
 					      NULL));
 
 	entry_type->get_playback_uri_data = source;
 
-	priv = MTP_SOURCE_GET_PRIVATE (source);
-	priv->device = device;
-	priv->udi = g_strdup (udi);
-
-	rb_mtp_source_load_tracks (source);
-
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
 
 	return RB_BROWSER_SOURCE (source);
@@ -276,19 +502,25 @@ entry_set_string_prop (RhythmDB *db,
 
 static void
 add_mtp_track_to_db (RBMtpSource *source,
+		     RhythmDB *db,
 		     LIBMTP_track_t *track)
 {
 	RhythmDBEntry *entry = NULL;
 	RhythmDBEntryType entry_type;
-	RhythmDB *db = NULL;
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
 	char *name = NULL;
 
-	db = get_db_for_source (source);
+	/* ignore everything except audio (allow audio/video types too, since they're probably pretty common) */
+	if (!(LIBMTP_FILETYPE_IS_AUDIO (track->filetype) || LIBMTP_FILETYPE_IS_AUDIOVIDEO (track->filetype))) {
+		rb_debug ("ignoring non-audio item %d (filetype %s)",
+			  track->item_id,
+			  LIBMTP_Get_Filetype_Description (track->filetype));
+		return;
+	}
 
 	/* Set URI */
 	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
-	name = g_strdup_printf ("rb-mtp-%i", track->item_id);
+	name = g_strdup_printf ("x-rb-mtp://%i/%s", track->item_id, track->filename);
 	entry = rhythmdb_entry_new (RHYTHMDB (db), entry_type, name);
 	g_free (name);
         g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
@@ -332,6 +564,27 @@ add_mtp_track_to_db (RBMtpSource *source,
 		g_value_unset (&value);
 	}
 
+	/* Set playcount */
+	if (track->usecount != 0) {
+		GValue value = {0, };
+		g_value_init (&value, G_TYPE_ULONG);
+		g_value_set_ulong (&value, track->usecount);
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_PLAY_COUNT,
+					       &value);
+		g_value_unset (&value);
+	}
+	/* Set rating */
+	if (track->rating != 0) {
+		GValue value = {0, };
+		g_value_init (&value, G_TYPE_DOUBLE);
+		g_value_set_double (&value, track->rating/20);
+		rhythmdb_entry_set (RHYTHMDB (db), entry,
+					       RHYTHMDB_PROP_RATING,
+					       &value);
+		g_value_unset (&value);
+	}
+
 	/* Set title */
 	entry_set_string_prop (RHYTHMDB (db), entry, RHYTHMDB_PROP_TITLE, track->title);
 
@@ -342,8 +595,6 @@ add_mtp_track_to_db (RBMtpSource *source,
 
 	g_hash_table_insert (priv->entry_map, entry, track);
 	rhythmdb_commit (RHYTHMDB (db));
-
-	g_object_unref (G_OBJECT (db));
 }
 
 static gboolean
@@ -357,12 +608,16 @@ load_mtp_db_idle_cb (RBMtpSource* source)
 
 	g_assert (db != NULL);
 
+#ifdef HAVE_LIBMTP_030
+	tracks = LIBMTP_Get_Tracklisting_With_Callback (priv->device, NULL, NULL);
+#else
 	tracks = LIBMTP_Get_Tracklisting (priv->device);
+#endif
+	report_libmtp_errors (priv->device, FALSE);
 	if (tracks != NULL) {
-		LIBMTP_track_t *track, *tmp = NULL;
+		LIBMTP_track_t *track;
 		for (track = tracks; track != NULL; track = track->next) {
-			add_mtp_track_to_db (source, track);
-			tmp = track;
+			add_mtp_track_to_db (source, db, track);
 		}
 	} else {
 		rb_debug ("No tracks");
@@ -378,57 +633,24 @@ rb_mtp_source_load_tracks (RBMtpSource *source)
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
 	char *name = NULL;
 
-	if ((priv->device != NULL) && (priv->entry_map != NULL)) {
-		name = LIBMTP_Get_Friendlyname (priv->device);
-		if (name == NULL) {
-			name = LIBMTP_Get_Modelname (priv->device);
-		}
-		if (name == NULL) {
-			name = g_strdup (_("Digital Audio Player"));
-		}
-
-		g_object_set (RB_SOURCE (source),
-			      "name", name,
-			      NULL);
-		g_idle_add ((GSourceFunc)load_mtp_db_idle_cb, source);
+	name = LIBMTP_Get_Friendlyname (priv->device);
+	/* ignore some particular broken device names */
+	if (name == NULL || strcmp (name, "?????") == 0) {
+		g_free (name);
+		name = LIBMTP_Get_Modelname (priv->device);
 	}
+	if (name == NULL) {
+		name = g_strdup (_("Digital Audio Player"));
+	}
+
+	g_object_set (RB_SOURCE (source),
+		      "name", name,
+		      NULL);
+
+	priv->load_songs_idle_id = g_idle_add ((GSourceFunc)load_mtp_db_idle_cb, source);
 	g_free (name);
 }
 
-static gboolean
-destroy_entry_map_pair (RhythmDBEntry *entry, LIBMTP_track_t *track, RhythmDB *db)
-{
-	LIBMTP_destroy_track_t (track);
-	rhythmdb_entry_delete (db, entry);
-	return TRUE;
-}
-
-static void
-impl_delete_thyself (RBSource *asource)
-{
-	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (asource);
-	RhythmDB *db = get_db_for_source (RB_MTP_SOURCE (asource));
-
-	if (priv->entry_map) {
-		g_hash_table_foreach_remove (priv->entry_map, (GHRFunc)destroy_entry_map_pair, db);
-		g_hash_table_destroy (priv->entry_map);
-		rhythmdb_commit (db);
-		priv->entry_map = NULL;
-	}
-
-	g_object_unref (db);
-
-	LIBMTP_Release_Device (priv->device);
-	priv->device = NULL;
-
-	if (priv->udi != NULL) {
-		g_free (priv->udi);
-		priv->udi = NULL;
-	}
-
-	RB_SOURCE_CLASS (rb_mtp_source_parent_class)->impl_delete_thyself (asource);
-}
-
 static char *
 gdate_to_char (GDate* date)
 {
@@ -439,79 +661,73 @@ gdate_to_char (GDate* date)
 }
 
 static LIBMTP_filetype_t
-mimetype_to_filetype (const char *mimetype)
+mimetype_to_filetype (RBMtpSource *source, const char *mimetype)
 {
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+
 	if (!strcmp (mimetype, "audio/mpeg") || !strcmp (mimetype, "application/x-id3")) {
 		return LIBMTP_FILETYPE_MP3;
 	}  else if (!strcmp (mimetype, "audio/x-wav")) {
 		return  LIBMTP_FILETYPE_WAV;
 	} else if (!strcmp (mimetype, "application/ogg")) {
 		return LIBMTP_FILETYPE_OGG;
-	} else if (!strcmp (mimetype, "audio/mp4")) {
-		return LIBMTP_FILETYPE_MP4;
+	} else if (!strcmp (mimetype, "audio/x-m4a") || !strcmp (mimetype, "video/quicktime")) {
+		/* try a few different filetypes that might work */
+		if (priv->supported_types[LIBMTP_FILETYPE_M4A])
+			return LIBMTP_FILETYPE_M4A;
+		else if (!priv->supported_types[LIBMTP_FILETYPE_AAC] && priv->supported_types[LIBMTP_FILETYPE_MP4])
+			return LIBMTP_FILETYPE_MP4;
+		else
+			return LIBMTP_FILETYPE_AAC;
+
 	} else if (!strcmp (mimetype, "audio/x-ms-wma") || !strcmp (mimetype, "audio/x-ms-asf")) {
 		return LIBMTP_FILETYPE_WMA;
+	} else if (!strcmp (mimetype, "video/x-ms-asf")) {
+		return LIBMTP_FILETYPE_ASF;
+	} else if (!strcmp (mimetype, "audio/x-flac")) {
+		return LIBMTP_FILETYPE_FLAC;
 	} else {
 		rb_debug ("\"%s\" is not a supported mimetype", mimetype);
 		return LIBMTP_FILETYPE_UNKNOWN;
 	}
 }
 
-static const char*
-filetype_to_mimetype (LIBMTP_filetype_t filetype)
-{
-	if (filetype == LIBMTP_FILETYPE_WAV) {
-		return "audio/x-wav";
-	} else if (filetype == LIBMTP_FILETYPE_MP3) {
-		return "audio/mpeg";
-	} else if (filetype == LIBMTP_FILETYPE_WMA) {
-		return "audio/x-ms-wma";
-	} else if (filetype == LIBMTP_FILETYPE_OGG) {
-		return "application/ogg";
-	} else if (filetype == LIBMTP_FILETYPE_MP4) {
-		return "audio/mp4";
-	} else if (filetype == LIBMTP_FILETYPE_WMV) {
-		return "audio/x-ms-wmv";
-	} else {
-		return NULL;
-	}
-}
-
 static void
 impl_delete (RBSource *source)
 {
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
-	GList *sel = NULL;
-	GList *tem = NULL;
-	RBEntryView *tracks = NULL;
-	RhythmDB *db = NULL;
-	RhythmDBEntry *entry = NULL;
-	const gchar *uri = NULL;
-	LIBMTP_track_t *track = NULL;
-	int ret = -1;
+	GList *sel;
+	GList *tem;
+	RBEntryView *tracks;
+	RhythmDB *db;
+	int ret;
 
 	db = get_db_for_source (RB_MTP_SOURCE (source));
 
 	tracks = rb_source_get_entry_view (source);
 	sel = rb_entry_view_get_selected_entries (tracks);
 	for (tem = sel; tem != NULL; tem = tem->next) {
+		LIBMTP_track_t *track;
+		RhythmDBEntry *entry;
+		const char *uri;
 
 		entry = (RhythmDBEntry *)tem->data;
 		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 		track = g_hash_table_lookup (priv->entry_map, entry);
 		if (track == NULL) {
-			rb_debug ("Couldn't find track on mtp-device! (%s)\n", uri);
+			rb_debug ("Couldn't find track on mtp-device! (%s)", uri);
 			continue;
 		}
 
 		ret = LIBMTP_Delete_Object (priv->device, track->item_id);
-		if (ret == 0) {
-			g_hash_table_remove (priv->entry_map, entry);
-			LIBMTP_destroy_track_t (track);
-			rhythmdb_entry_delete (db, entry);
-		} else {
-			rb_debug ("Delete track failed");
+		if (ret != 0) {
+			rb_debug ("Delete track %d failed", track->item_id);
+			report_libmtp_errors (priv->device, TRUE);
+			continue;
 		}
+
+		g_hash_table_remove (priv->entry_map, entry);
+		rhythmdb_entry_delete (db, entry);
 	}
 	rhythmdb_commit (db);
 
@@ -536,74 +752,37 @@ impl_get_ui_actions (RBSource *source)
 	return actions;
 }
 
-gboolean
-rb_mtp_source_is_udi (RBMtpSource *source,
-		      const char *udi)
-{
-	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
-
-	return (strcmp (udi, priv->udi) == 0);
-}
-
-/*Borowed from rb-playlist-source-recorder*/
-static gboolean
-check_dir_has_space (const char *path,
-		     guint64 bytes_needed)
-{
-	GnomeVFSResult result;
-	GnomeVFSURI *dir_uri = NULL;
-	GnomeVFSFileSize free_bytes;
-
-	if (!g_file_test (path, G_FILE_TEST_IS_DIR))
-		return FALSE;
-
-	dir_uri = gnome_vfs_uri_new (path);
-	if (dir_uri == NULL) {
-		rb_debug ("Cannot get free space at %s\n", path);
-		return FALSE;
-	}
-
-	result = gnome_vfs_get_volume_free_space (dir_uri, &free_bytes);
-	gnome_vfs_uri_unref (dir_uri);
-
-	if (result != GNOME_VFS_OK) {
-		rb_debug ("Cannot get free space at %s\n", path);
-		return FALSE;
-	}
-
-	if (bytes_needed >= free_bytes) {
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 static gboolean
 rb_mtp_source_transfer_track_to_disk (LIBMTP_mtpdevice_t *device,
 				      LIBMTP_track_t *track,
 				      const char *uri)
 {
 	int ret = -1;
-	GnomeVFSURI* guri = NULL;
+	char *path;
 
 	if (device == NULL || track == NULL || strlen (uri) == 0) {
+		rb_debug ("device (%p), track (%p), or URI (%s) not supplied", device, track, uri);
 		return FALSE;
 	}
 
-	guri = gnome_vfs_uri_new (uri);
-	if (!check_dir_has_space (gnome_vfs_uri_get_path (gnome_vfs_uri_get_parent (guri)), track->filesize)) {
-		gnome_vfs_uri_unref (guri);
+	if (rb_check_dir_has_space_uri (uri, track->filesize) == FALSE) {
+		rb_debug ("not enough space to transfer track %d to %s", track->item_id, uri);
 		return FALSE;
 	}
-	gnome_vfs_uri_unref (guri);
 
-	ret = LIBMTP_Get_Track_To_File (device, track->item_id, uri, NULL, NULL);
-
-	if (ret == 0) {
-		return TRUE;
+	path = g_filename_from_uri (uri, NULL, NULL);
+	if (path != NULL) {
+		ret = LIBMTP_Get_Track_To_File (device, track->item_id, path, NULL, NULL);
+		rb_debug ("LIBMTP_Get_Track_To_File(%d, %s) returned %d", track->item_id, path, ret);
+		if (ret != 0) {
+			report_libmtp_errors (device, TRUE);
+		}
+		g_free (path);
 	} else {
-		return FALSE;
+		g_warning ("couldn't get path from URI %s", uri);
 	}
+
+	return (ret == 0);
 }
 
 static char *
@@ -611,20 +790,31 @@ rb_mtp_source_get_playback_uri (RhythmDBEntry *entry, gpointer data)
 {
 	RBMtpSourcePrivate *priv;
 	LIBMTP_track_t *track;
+	char *path;
 	char *uri = NULL;
-	char *s = NULL;
+	GError *error = NULL;
 
 	priv = MTP_SOURCE_GET_PRIVATE (data);
 
 	track = g_hash_table_lookup (priv->entry_map, entry);
-	uri = g_strdup_printf ("%s/%s-%s", g_get_tmp_dir (),
-			       rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST),
-			       rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE));
+	path = g_strdup_printf ("%s/%s-%s",
+				g_get_tmp_dir (),
+				rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST),
+				rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE));
+	uri = g_filename_to_uri (path, NULL, &error);
+	g_free (path);
+	if (error != NULL) {
+		g_warning ("unable to convert path %s to filename: %s", path, error->message);
+		g_error_free (error);
+		g_free (path);
+		return NULL;
+	}
 
 	if (rb_mtp_source_transfer_track_to_disk (priv->device, track, uri) == TRUE) {
-		s = g_strdup_printf ("file://%s", uri);
-		g_free (uri);
-		return s;
+		rb_debug ("playback URI for %s: %s",
+			  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION),
+			  uri);
+		return uri;
 	} else {
 		g_free (uri);
 		return NULL;
@@ -640,7 +830,6 @@ impl_copy (RBSource *source)
 	GList *iter;
 	GList *copy_entries;
 	int ret = -1;
-	const char *uri = NULL;
 	LIBMTP_track_t *track = NULL;
 
 	db = get_db_for_source (RB_MTP_SOURCE (source));
@@ -649,6 +838,8 @@ impl_copy (RBSource *source)
 	selected_entries = rb_entry_view_get_selected_entries (rb_source_get_entry_view (source));
 	for (iter = selected_entries; iter != NULL; iter = g_list_next (iter)) {
 		RhythmDBEntry *entry;
+		char *path;
+		char *uri;
 
 		entry = (RhythmDBEntry *)iter->data;
 		track = g_hash_table_lookup (priv->entry_map, entry);
@@ -656,13 +847,16 @@ impl_copy (RBSource *source)
 		if (track == NULL)
 			continue;
 
-		uri = g_strdup_printf ("%s/%s", g_get_tmp_dir (), track->filename);
+		path = g_strdup_printf ("%s/%s", g_get_tmp_dir (), track->filename);
+		uri = g_filename_to_uri (path, NULL, NULL);
+		g_free (path);
 		ret = rb_mtp_source_transfer_track_to_disk (priv->device, track, uri);
 
 		if (ret == 0) {
-			entry_set_string_prop (RHYTHMDB (db), entry, RHYTHMDB_PROP_LOCATION, g_strdup_printf ("file://%s", uri));
+			entry_set_string_prop (RHYTHMDB (db), entry, RHYTHMDB_PROP_LOCATION, uri);
 			copy_entries = g_list_prepend (copy_entries, entry);
 		}
+		g_free (uri);
 	}
 
 	g_list_free (selected_entries);
@@ -694,6 +888,7 @@ transfer_track (RBMtpSource *source,
 {
 	LIBMTP_track_t *trackmeta = LIBMTP_new_track_t ();
 	GDate d;
+	int ret;
 
 	trackmeta->title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
 	trackmeta->album = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ALBUM);
@@ -707,16 +902,27 @@ transfer_track (RBMtpSource *source,
 	}
 	trackmeta->tracknumber = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER);
 	trackmeta->duration = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION) * 1000;
+	trackmeta->rating = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_RATING) * 20;
+	trackmeta->usecount = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_PLAY_COUNT);
 	trackmeta->filesize = filesize;
 	if (mimetype == NULL) {
-		trackmeta->filetype = mimetype_to_filetype (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE));
-	} else {
-		trackmeta->filetype = mimetype_to_filetype (mimetype);
+		mimetype = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE);
 	}
+	trackmeta->filetype = mimetype_to_filetype (source, mimetype);
+	rb_debug ("using libmtp filetype %d (%s) for source media type %s",
+		  trackmeta->filetype,
+		  LIBMTP_Get_Filetype_Description (trackmeta->filetype),
+		  mimetype);
 
-	if (LIBMTP_Send_Track_From_File (device, filename, trackmeta, NULL, NULL, 0) != 0) {
+#ifdef HAVE_LIBMTP_030
+	ret = LIBMTP_Send_Track_From_File (device, filename, trackmeta, NULL, NULL);
+#else
+	ret = LIBMTP_Send_Track_From_File (device, filename, trackmeta, NULL, NULL, 0);
+#endif
+	rb_debug ("LIBMTP_Send_Track_From_File (%s) returned %d", filename, ret);
+	if (ret != 0) {
+		report_libmtp_errors (device, TRUE);
 		LIBMTP_destroy_track_t (trackmeta);
-		rb_debug ("Tracktransfer failed\n");
 		return NULL;
 	}
 
@@ -727,24 +933,7 @@ static GList *
 impl_get_mime_types (RBRemovableMediaSource *source)
 {
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
-	guint16 *types = NULL;
-	guint16 num_devices = 0;
-	int i = 0;
-	GList *list = NULL;
-
-	if (LIBMTP_Get_Supported_Filetypes (priv->device, &types, &num_devices) == 0) {
-		for (i = 0; i < num_devices; i++) {
-			const char *mime = filetype_to_mimetype (types[i]);
-
-			if (mime != NULL) {
-				list = g_list_prepend (list, g_strdup (mime));
-			}
-		}
-	} else {
-		rb_debug ("Get supported filetypes failed");
-	}
-
-	return list;
+	return rb_string_list_copy (priv->mediatypes);
 }
 
 static gboolean
@@ -756,17 +945,24 @@ impl_track_added (RBRemovableMediaSource *isource,
 {
 	RBMtpSource *source = RB_MTP_SOURCE (isource);
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
-	char *filename = NULL;
+	GFile *file;
+	char *path;
 	LIBMTP_track_t *track = NULL;
 
-	filename = g_filename_from_uri (dest, NULL, NULL);
-	track = transfer_track (source, priv->device, entry, filename, filesize, mimetype);
-	gnome_vfs_unlink (filename);
-	g_free (filename);
+	file = g_file_new_for_uri (dest);
+	path = g_file_get_path (file);
+	track = transfer_track (source, priv->device, entry, path, filesize, mimetype);
+	g_free (path);
+
+	g_file_delete (file, NULL, NULL);
 
 	if (track != NULL) {
+		RhythmDB *db;
 		/*request_artwork (isource, entry, song);*/
-		add_mtp_track_to_db (source, track);
+
+		db = get_db_for_source (source);
+		add_mtp_track_to_db (source, db, track);
+		g_object_unref (db);
 	}
 
 	return FALSE;
