@@ -8,6 +8,14 @@
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  *
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -348,12 +356,10 @@ can_draw_on_desktop (RBVisualizerPlugin *plugin, int screen)
 	/* if we have a compositing manager, we probably can't just
 	 * scribble on the desktop.
 	 */
-#if GTK_CHECK_VERSION(2,10,0)
 	if (gdk_screen_is_composited (gdk_display_get_screen (display, screen))) {
 		rb_debug ("screen is composited: probably can't draw on desktop");
 		return FALSE;
 	}
-#endif
 
 	selection_name = g_strdup_printf ("_NET_DESKTOP_MANAGER_S%d", screen);
 	selection_atom = gdk_atom_intern (selection_name, FALSE);
@@ -1320,7 +1326,7 @@ totem_display_is_local (void)
 
 	/* Get to the character after the colon */
 	work++;
-	if (work == NULL)
+	if (*work == '\0')
 		return TRUE;
 
 	if (sscanf (work, "%d.%d", &display, &screen) != 2)
