@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -46,8 +46,6 @@
 #include <libxml/tree.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
 
 #include "rb-playlist-manager.h"
 #include "rb-playlist-source.h"
@@ -87,6 +85,8 @@ static void rb_playlist_manager_cmd_load_playlist (GtkAction *action,
 						   RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 						   RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_save_queue (GtkAction *action,
+						RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_new_playlist (GtkAction *action,
 						  RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
@@ -144,6 +144,26 @@ enum
 
 static guint rb_playlist_manager_signals[LAST_SIGNAL] = { 0 };
 
+typedef struct {
+  const gchar *description;
+  /* NULL terminated array of extensions for this file format.  The first
+   * one is the prefered extension for files of this type. */
+  const gchar **extensions;
+  const char *mimetype;
+  const RBPlaylistExportType type;
+} RBPlaylistExportFilter;
+
+static const char *m3u_extensions [] = {"m3u", NULL};
+static const char *pls_extensions [] = {"pls", NULL};
+static const char *xspf_extensions[] = {"xspf", NULL};
+
+static RBPlaylistExportFilter playlist_formats[] = {
+	{N_("MPEG Version 3.0 URL"), m3u_extensions, "audio/x-mpegurl", RB_PLAYLIST_EXPORT_TYPE_M3U},
+	{N_("Shoutcast playlist"), pls_extensions, "audio/x-scpls", RB_PLAYLIST_EXPORT_TYPE_PLS},
+	{N_("XML Shareable Playlist Format"), xspf_extensions, "application/xspf+xml", RB_PLAYLIST_EXPORT_TYPE_XSPF},
+};
+
+
 static GtkActionEntry rb_playlist_manager_actions [] =
 {
 	/* Submenu of Music */
@@ -176,6 +196,9 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	{ "ShufflePlaylist", NULL, N_("_Shuffle Playlist"), NULL,
 	  N_("Shuffle the tracks in this playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_shuffle_playlist) },
+	{ "MusicPlaylistSaveQueue", GTK_STOCK_SAVE_AS, N_("_Save to File..."), NULL,
+	  N_("Save the play queue to a file"),
+	  G_CALLBACK (rb_playlist_manager_cmd_save_queue) },
 };
 static guint rb_playlist_manager_n_actions = G_N_ELEMENTS (rb_playlist_manager_actions);
 
@@ -1124,7 +1147,7 @@ static void
 rb_playlist_manager_cmd_new_playlist (GtkAction *action,
 				      RBPlaylistManager *mgr)
 {
-	rb_playlist_manager_new_playlist (mgr, NULL, FALSE);
+	rb_playlist_manager_new_playlist (mgr, _("New Playlist"), FALSE);
 }
 
 static void
@@ -1167,7 +1190,7 @@ rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
 		return;
 	}
 
-	playlist = rb_playlist_manager_new_playlist (mgr, NULL, TRUE);
+	playlist = rb_playlist_manager_new_playlist (mgr, _("New Playlist"), TRUE);
 
 	rb_playlist_manager_set_automatic_playlist (mgr, RB_AUTO_PLAYLIST_SOURCE (playlist), creator);
 
@@ -1368,36 +1391,37 @@ rb_playlist_manager_cmd_load_playlist (GtkAction *action,
 				       RBPlaylistManager *mgr)
 {
 	GtkWidget *dialog;
+	GtkFileFilter *filter;
+	GtkFileFilter *filter_all;
+	int i;
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("Playlists"));
+	for (i = 0; i < G_N_ELEMENTS (playlist_formats); i++) {
+		gtk_file_filter_add_mime_type (filter, playlist_formats[i].mimetype);
+	}
+
+	filter_all = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter_all, _("All Files"));
+	gtk_file_filter_add_pattern (filter_all, "*");
+
 
 	dialog = rb_file_chooser_new (_("Load Playlist"),
 				      GTK_WINDOW (mgr->priv->window),
 				      GTK_FILE_CHOOSER_ACTION_OPEN,
 				      FALSE);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter_all);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
 
 	g_signal_connect_object (dialog, "response",
 				 G_CALLBACK (load_playlist_response_cb), mgr, 0);
 }
 
-typedef struct {
-  const gchar *description;
-  /* NULL terminated array of extensions for this file format.  The first
-   * one is the prefered extension for files of this type. */
-  const gchar **extensions;
-  const RBPlaylistExportType type;
-} RBPlaylistExportFilter;
-
-static const char *m3u_extensions [] = {"m3u", NULL};
-static const char *pls_extensions [] = {"pls", NULL};
-
-static RBPlaylistExportFilter playlist_export_formats[] = {
-	{N_("MPEG Version 3.0 URL"), m3u_extensions, RB_PLAYLIST_EXPORT_TYPE_M3U},
-	{N_("Shoutcast playlist"), pls_extensions, RB_PLAYLIST_EXPORT_TYPE_PLS},
-};
-
 static void
 save_playlist_response_cb (GtkDialog *dialog,
 			   int response_id,
-			   RBPlaylistManager *mgr)
+			   RBSource *source)
 {
 	char *file = NULL;
 	GtkWidget *menu;
@@ -1420,26 +1444,26 @@ save_playlist_response_cb (GtkDialog *dialog,
 	if (index <= 0) {
 		int i;
 
-		for (i = 0; i < G_N_ELEMENTS (playlist_export_formats); i++) {
+		for (i = 0; i < G_N_ELEMENTS (playlist_formats); i++) {
 			int j;
 
 			/* determine the playlist type from the extension */
-			for (j = 0; playlist_export_formats[i].extensions[j] != NULL; j++) {
-				if (g_str_has_suffix (file, playlist_export_formats[i].extensions[j])) {
-					export_type = playlist_export_formats[i].type;
+			for (j = 0; playlist_formats[i].extensions[j] != NULL; j++) {
+				if (g_str_has_suffix (file, playlist_formats[i].extensions[j])) {
+					export_type = playlist_formats[i].type;
 					break;
 				}
 			}
 		}
 	} else {
-		export_type = playlist_export_formats[index-1].type;
+		export_type = playlist_formats[index-1].type;
 	}
 
 	if (export_type == RB_PLAYLIST_EXPORT_TYPE_UNKNOWN) {
-			rb_error_dialog (NULL, _("Couldn't save playlist"), _("Unsupported file extension given."));
+		rb_error_dialog (NULL, _("Couldn't save playlist"), _("Unsupported file extension given."));
 	} else {
-		rb_playlist_source_save_playlist (RB_PLAYLIST_SOURCE (mgr->priv->selected_source),
-						  file, (export_type == RB_PLAYLIST_EXPORT_TYPE_M3U));
+		rb_playlist_source_save_playlist (RB_PLAYLIST_SOURCE (source),
+						  file, export_type);
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	}
 
@@ -1460,7 +1484,7 @@ export_set_extension_cb (GtkWidget* widget, GtkDialog *dialog)
 	if (index <= 0)
 		return;
 
-	extension = playlist_export_formats[index-1].extensions[0];
+	extension = playlist_formats[index-1].extensions[0];
 	if (extension == NULL)
 		return;
 
@@ -1514,11 +1538,11 @@ setup_format_menu (GtkWidget* menu, GtkWidget *dialog)
 	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (menu), rb_combo_box_hyphen_separator_func,
 					      NULL, NULL);
 
-	for (i = 0; i < G_N_ELEMENTS (playlist_export_formats); i++) {
+	for (i = 0; i < G_N_ELEMENTS (playlist_formats); i++) {
 		gchar *filter_label;
 		GtkTreeIter iter;
 
-		filter_label = filter_get_export_filter_label (&playlist_export_formats[i]);
+		filter_label = filter_get_export_filter_label (&playlist_formats[i]);
 		gtk_list_store_insert_with_values (GTK_LIST_STORE (model), &iter, -1,
 						   0, filter_label, -1);
 
@@ -1531,8 +1555,7 @@ setup_format_menu (GtkWidget* menu, GtkWidget *dialog)
 }
 
 static void
-rb_playlist_manager_cmd_save_playlist (GtkAction *action,
-				       RBPlaylistManager *mgr)
+save_playlist (RBPlaylistManager *mgr, RBSource *source)
 {
 	GladeXML *xml;
 	GtkWidget *dialog, *menu;
@@ -1547,7 +1570,7 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 	setup_format_menu (menu, dialog);
 	g_object_set_data (G_OBJECT (dialog), "export-menu", menu);
 
-	g_object_get (mgr->priv->selected_source, "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 	tmp = g_strconcat (name, ".pls", NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), tmp);
 	g_free (tmp);
@@ -1557,11 +1580,28 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 	gtk_combo_box_set_active (GTK_COMBO_BOX (menu), 0);
 	g_signal_connect_object (dialog, "response",
 				 G_CALLBACK (save_playlist_response_cb),
-				 mgr, 0);
+				 source, 0);
 
 	g_object_unref (xml);
 }
 
+static void
+rb_playlist_manager_cmd_save_playlist (GtkAction *action,
+				       RBPlaylistManager *mgr)
+{
+	save_playlist (mgr, mgr->priv->selected_source);
+}
+
+static void
+rb_playlist_manager_cmd_save_queue (GtkAction *action,
+				    RBPlaylistManager *mgr)
+{
+	RBSource *queue;
+	g_object_get (mgr->priv->shell, "queue-source", &queue, NULL);
+	save_playlist (mgr, queue);
+	g_object_unref (queue);
+}
+
 static gboolean
 list_playlists_cb (GtkTreeModel *model,
 		   GtkTreePath  *path,
