@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -36,6 +44,21 @@
 #include "rb-shell-player.h"
 #include "eel-gconf-extensions.h"
 #include "rb-util.h"
+#include "rhythmdb.h"
+
+/**
+ * SECTION:rb-header
+ * @short_description: playback area widgetry
+ *
+ * The RBHeader widget displays information about the current playing track
+ * (title, album, artist), the elapsed or remaining playback time, and a
+ * position slider indicating the playback position.  It translates slider
+ * move and drag events into seek requests for the player backend.
+ *
+ * For shoutcast-style streams, the title/artist/album display is supplemented
+ * by metadata extracted from the stream.  See #RBStreamingSource for more information
+ * on how the metadata is reported.
+ */
 
 static void rb_header_class_init (RBHeaderClass *klass);
 static void rb_header_init (RBHeader *header);
@@ -51,10 +74,12 @@ static void rb_header_get_property (GObject *object,
 static void rb_header_set_show_timeline (RBHeader *header,
 			                 gboolean show);
 static void rb_header_update_elapsed (RBHeader *header);
+static void apply_slider_position (RBHeader *header);
 static gboolean slider_press_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *header);
 static gboolean slider_moved_callback (GtkWidget *widget, GdkEventMotion *event, RBHeader *header);
 static gboolean slider_release_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *header);
 static void slider_changed_callback (GtkWidget *widget, RBHeader *header);
+static gboolean slider_scroll_callback (GtkWidget *widget, GdkEventScroll *event, RBHeader *header);
 
 static void rb_header_elapsed_changed_cb (RBShellPlayer *player, guint elapsed, RBHeader *header);
 
@@ -63,8 +88,6 @@ struct RBHeaderPrivate
 	RhythmDB *db;
 	RhythmDBEntry *entry;
 
-	char *title;
-
 	RBShellPlayer *shell_player;
 
 	GtkWidget *image;
@@ -80,10 +103,9 @@ struct RBHeaderPrivate
 	gboolean slider_locked;
 	guint slider_moved_timeout;
 	long latest_set_time;
-	guint value_changed_update_handler;
 	GtkWidget *elapsed;
 
-	long elapsed_time;
+	guint elapsed_time;
 	long duration;
 	gboolean seekable;
 };
@@ -94,12 +116,17 @@ enum
 	PROP_DB,
 	PROP_SHELL_PLAYER,
 	PROP_ENTRY,
-	PROP_TITLE,
 	PROP_SEEKABLE,
+	PROP_SLIDER_DRAGGING
 };
 
-#define SONG_MARKUP(xSONG) g_markup_printf_escaped ("<big><b>%s</b></big>", xSONG);
-#define SONG_MARKUP_ALBUM_ARTIST(xSONG, xALBUM, xARTIST) g_markup_printf_escaped ("<big><b>%s</b></big> %s <i>%s</i> %s <i>%s</i>", xSONG, _("by"), xARTIST, _("from"), xALBUM);
+#define TITLE_MARKUP(xTITLE) g_markup_printf_escaped ("<big><b>%s</b></big>", xTITLE)
+#define ALBUM_MARKUP(xALBUM) g_markup_printf_escaped (" %s <i>%s</i>", _("from"), xALBUM)
+#define ARTIST_MARKUP(xARTIST) g_markup_printf_escaped (" %s <i>%s</i>", _("by"), xARTIST)
+#define STREAM_MARKUP(xSTREAM) g_markup_printf_escaped (" (%s)", xSTREAM)
+
+#define SCROLL_UP_SEEK_OFFSET	5
+#define SCROLL_DOWN_SEEK_OFFSET -5
 
 G_DEFINE_TYPE (RBHeader, rb_header, GTK_TYPE_HBOX)
 
@@ -113,6 +140,11 @@ rb_header_class_init (RBHeaderClass *klass)
 	object_class->set_property = rb_header_set_property;
 	object_class->get_property = rb_header_get_property;
 
+	/**
+	 * RBHeader:db:
+	 *
+	 * #RhythmDB instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_DB,
 					 g_param_spec_object ("db",
@@ -121,6 +153,11 @@ rb_header_class_init (RBHeaderClass *klass)
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE));
 
+	/**
+	 * RBHeader:entry:
+	 *
+	 * The #RhythmDBEntry to display
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY,
 					 g_param_spec_boxed ("entry",
@@ -128,6 +165,11 @@ rb_header_class_init (RBHeaderClass *klass)
 							     "RhythmDBEntry pointer",
 							     RHYTHMDB_TYPE_ENTRY,
 							     G_PARAM_READWRITE));
+	/**
+	 * RBHeader:shell-player:
+	 *
+	 * The #RBShellPlayer instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SHELL_PLAYER,
 					 g_param_spec_object ("shell-player",
@@ -135,14 +177,11 @@ rb_header_class_init (RBHeaderClass *klass)
 							      "RBShellPlayer object",
 							      RB_TYPE_SHELL_PLAYER,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (object_class,
-					 PROP_TITLE,
-					 g_param_spec_string ("title",
-							      "title",
-							      "title",
-							      NULL,
-							      G_PARAM_READWRITE));
-
+	/**
+	 * RBHeader:seekable:
+	 *
+	 * If TRUE, the header should allow seeking by dragging the playback position slider
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SEEKABLE,
 					 g_param_spec_boolean ("seekable",
@@ -151,13 +190,26 @@ rb_header_class_init (RBHeaderClass *klass)
 							       TRUE,
 							       G_PARAM_READWRITE));
 
+	/**
+	 * RBHeader:slider-dragging:
+	 *
+	 * Whether the song position slider is currently being dragged.
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_SLIDER_DRAGGING,
+					 g_param_spec_boolean ("slider-dragging",
+						 	       "slider dragging",
+							       "slider dragging",
+							       FALSE,
+							       G_PARAM_READABLE));
+
 	g_type_class_add_private (klass, sizeof (RBHeaderPrivate));
 }
 
 static void
 rb_header_init (RBHeader *header)
 {
-	/**
+	/*
 	 * The children in this widget look like this:
 	 * RBHeader
 	 *   GtkHBox
@@ -187,6 +239,7 @@ rb_header_init (RBHeader *header)
  	gtk_label_set_use_markup (GTK_LABEL (header->priv->song), TRUE);
  	gtk_label_set_selectable (GTK_LABEL (header->priv->song), TRUE);
 	gtk_label_set_ellipsize (GTK_LABEL (header->priv->song), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (header->priv->song), 0.0, 0.0);
 	gtk_box_pack_start (GTK_BOX (hbox), header->priv->song, TRUE, TRUE, 0);
 	gtk_widget_show (header->priv->song);
 
@@ -196,7 +249,6 @@ rb_header_init (RBHeader *header)
 
 	gtk_misc_set_padding (GTK_MISC (header->priv->elapsed), 2, 0);
 	gtk_box_pack_start (GTK_BOX (header->priv->timeline), header->priv->elapsed, FALSE, FALSE, 0);
-	gtk_widget_set_sensitive (header->priv->timeline, FALSE);
 	gtk_box_pack_end (GTK_BOX (hbox), header->priv->timeline, FALSE, FALSE, 0);
 	gtk_widget_show_all (header->priv->timeline);
 
@@ -223,6 +275,10 @@ rb_header_init (RBHeader *header)
 				 "value_changed",
 				 G_CALLBACK (slider_changed_callback),
 				 header, 0);
+	g_signal_connect_object (G_OBJECT (header->priv->scale),
+				 "scroll_event",
+				 G_CALLBACK (slider_scroll_callback),
+				 header, 0);
 	gtk_scale_set_draw_value (GTK_SCALE (header->priv->scale), FALSE);
 	gtk_widget_set_size_request (header->priv->scale, 150, -1);
 	gtk_box_pack_start (GTK_BOX (header->priv->scaleline), header->priv->scale, TRUE, TRUE, 0);
@@ -242,6 +298,24 @@ rb_header_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_header_parent_class)->finalize (object);
 }
 
+static void
+rb_header_set_playing_entry_internal (RBHeader *header, RhythmDBEntry *entry)
+{
+	if (header->priv->entry == entry)
+		return;
+
+	header->priv->entry = entry;
+	if (header->priv->entry) {
+		header->priv->duration = rhythmdb_entry_get_ulong (header->priv->entry,
+								   RHYTHMDB_PROP_DURATION);
+	} else {
+		header->priv->duration = 0;
+	}
+
+	header->priv->adjustment->upper = header->priv->duration;
+	gtk_adjustment_changed (header->priv->adjustment);
+}
+
 static void
 rb_header_set_property (GObject *object,
 			guint prop_id,
@@ -255,13 +329,7 @@ rb_header_set_property (GObject *object,
 		header->priv->db = g_value_get_object (value);
 		break;
 	case PROP_ENTRY:
-		header->priv->entry = g_value_get_boxed (value);
-		if (header->priv->entry) {
-			header->priv->duration = rhythmdb_entry_get_ulong (header->priv->entry,
-									   RHYTHMDB_PROP_DURATION);
-		} else {
-			header->priv->duration = -1;
-		}
+		rb_header_set_playing_entry_internal (header, g_value_get_boxed (value));
 		break;
 	case PROP_SHELL_PLAYER:
 		header->priv->shell_player = g_value_get_object (value);
@@ -270,10 +338,6 @@ rb_header_set_property (GObject *object,
 				  (GCallback) rb_header_elapsed_changed_cb,
 				  header);
 		break;
-	case PROP_TITLE:
-		g_free (header->priv->title);
-		header->priv->title = g_value_dup_string (value);
-		break;
 	case PROP_SEEKABLE:
 		header->priv->seekable = g_value_get_boolean (value);
 		break;
@@ -301,26 +365,33 @@ rb_header_get_property (GObject *object,
 	case PROP_SHELL_PLAYER:
 		g_value_set_object (value, header->priv->shell_player);
 		break;
-	case PROP_TITLE:
-		g_value_set_string (value, header->priv->title);
-		break;
 	case PROP_SEEKABLE:
 		g_value_set_boolean (value, header->priv->seekable);
 		break;
+	case PROP_SLIDER_DRAGGING:
+		g_value_set_boolean (value, header->priv->slider_dragging);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+/**
+ * rb_header_new:
+ * @shell_player: the #RBShellPlayer instance
+ * @db: the #RhythmDB instance
+ *
+ * Return value: the header widget
+ */
 RBHeader *
-rb_header_new (RBShellPlayer *shell_player)
+rb_header_new (RBShellPlayer *shell_player, RhythmDB *db)
 {
 	RBHeader *header;
 
 	header = RB_HEADER (g_object_new (RB_TYPE_HEADER,
 					  "shell-player", shell_player,
-					  "title", NULL,
+					  "db", db,
 					  "spacing", 6, NULL));
 
 	g_return_val_if_fail (header->priv != NULL, NULL);
@@ -328,6 +399,14 @@ rb_header_new (RBShellPlayer *shell_player)
 	return header;
 }
 
+/**
+ * rb_header_set_playing_entry:
+ * @header: the #RBHeader
+ * @entry: the new playing #RhythmDBEntry
+ * @seekable: whether the new playing entry is seekable
+ *
+ * Sets a new playing entry to be displayed in the header.
+ */
 void
 rb_header_set_playing_entry (RBHeader *header,
 			     RhythmDBEntry *entry,
@@ -339,48 +418,123 @@ rb_header_set_playing_entry (RBHeader *header,
 		      NULL);
 }
 
-void
-rb_header_set_title (RBHeader *header, const char *title)
+static void
+append_and_free (GString *str,
+		 char *text)
 {
-	g_object_set (header, "title", title, NULL);
+	g_string_append (str, text);
+	g_free (text);
 }
 
+static void
+get_extra_metadata (RhythmDB *db, RhythmDBEntry *entry, const char *field, char **value)
+{
+	GValue *v;
+
+	v = rhythmdb_entry_request_extra_metadata (db, entry, field);
+	if (v != NULL) {
+		*value = g_value_dup_string (v);
+		g_value_unset (v);
+		g_free (v);
+	} else {
+		*value = NULL;
+	}
+}
+
+/**
+ * rb_header_sync:
+ * @header: the #RBHeader
+ *
+ * Updates the header widget to be consistent with the current playing entry
+ * including all streaming metadata.
+ */
 void
 rb_header_sync (RBHeader *header)
 {
 	char *label_text;
-
-	rb_debug ("syncing with entry = %p", header->priv->entry);
+	const char *location = "<null>";
 
 	if (header->priv->entry != NULL) {
-		const char *song = header->priv->title;
+		location = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_LOCATION);
+	}
+	rb_debug ("syncing with entry = %s", location);
+
+	if (header->priv->entry != NULL) {
+		const char *title;
 		const char *album;
 		const char *artist;
+		const char *stream_name = NULL;
+		char *streaming_title;
+		char *streaming_artist;
+		char *streaming_album;
+		GString *label_str;
 
 		gboolean have_duration = (header->priv->duration > 0);
 
+		title = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_TITLE);
 		album = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_ALBUM);
 		artist = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_ARTIST);
 
-		/* check for artist and album */
-		if ((album != NULL && artist != NULL)
-		    && (strlen (album) > 0 && strlen (artist) > 0)) {
-			label_text = SONG_MARKUP_ALBUM_ARTIST (song, album, artist);
-		} else {
-			label_text = SONG_MARKUP (song);
+		get_extra_metadata (header->priv->db,
+				    header->priv->entry,
+				    RHYTHMDB_PROP_STREAM_SONG_TITLE,
+				    &streaming_title);
+		if (streaming_title) {
+			/* use entry title as stream name */
+			stream_name = title;
+			title = streaming_title;
 		}
 
+		get_extra_metadata (header->priv->db,
+				    header->priv->entry,
+				    RHYTHMDB_PROP_STREAM_SONG_ARTIST,
+				    &streaming_artist);
+		if (streaming_artist) {
+			/* override artist from entry */
+			artist = streaming_artist;
+		}
+
+		get_extra_metadata (header->priv->db,
+				    header->priv->entry,
+				    RHYTHMDB_PROP_STREAM_SONG_ALBUM,
+				    &streaming_album);
+		if (streaming_album) {
+			/* override album from entry */
+			album = streaming_album;
+		}
+
+		label_str = g_string_sized_new (100);
+
+		/* stick a right-to-left mark in the string for RTL display */
+		if (gtk_widget_get_direction (GTK_WIDGET (header->priv->song)) == GTK_TEXT_DIR_RTL) {
+			g_string_append (label_str, "\xE2\x80\x8F");
+		}
+
+		append_and_free (label_str, TITLE_MARKUP (title));
+
+		if (artist != NULL && artist[0] != '\0')
+			append_and_free (label_str, ARTIST_MARKUP (artist));
+
+		if (album != NULL && album[0] != '\0')
+			append_and_free (label_str, ALBUM_MARKUP (album));
+
+		if (stream_name && stream_name[0] != '\0')
+			append_and_free (label_str, STREAM_MARKUP (stream_name));
+
+		label_text = g_string_free (label_str, FALSE);
 		gtk_label_set_markup (GTK_LABEL (header->priv->song), label_text);
 		g_free (label_text);
 
 		rb_header_set_show_timeline (header, have_duration && header->priv->seekable);
 		if (have_duration)
 			rb_header_sync_time (header);
-	} else {
-		char *tmp;
 
+		g_free (streaming_artist);
+		g_free (streaming_album);
+		g_free (streaming_title);
+	} else {
 		rb_debug ("not playing");
-		label_text = SONG_MARKUP (_("Not Playing"));
+		label_text = TITLE_MARKUP (_("Not Playing"));
 		gtk_label_set_markup (GTK_LABEL (header->priv->song), label_text);
 		g_free (label_text);
 
@@ -391,12 +545,18 @@ rb_header_sync (RBHeader *header)
 		header->priv->slider_locked = FALSE;
 		gtk_widget_set_sensitive (header->priv->scale, FALSE);
 
-		tmp = rb_make_elapsed_time_string (0, 0, !eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY));
-		gtk_label_set_text (GTK_LABEL (header->priv->elapsed), tmp);
-		g_free (tmp);
+		gtk_label_set_text (GTK_LABEL (header->priv->elapsed), "");
 	}
 }
 
+/**
+ * rb_header_set_show_position_slider:
+ * @header: the #RBHeader
+ * @show: whether the position slider should be shown
+ *
+ * Sets the visibility of the position slider.  This is not currently
+ * used properly.
+ */
 void
 rb_header_set_show_position_slider (RBHeader *header,
 				    gboolean show)
@@ -418,34 +578,36 @@ static void
 rb_header_set_show_timeline (RBHeader *header,
 			     gboolean show)
 {
-	gtk_widget_set_sensitive (header->priv->timeline, show);
 	gtk_widget_set_sensitive (header->priv->scaleline, show);
 }
 
-gboolean
+/**
+ * rb_header_sync_time:
+ * @header: the #RBHeader
+ *
+ * Updates the time display components of the header.
+ * If the position slider is being dragged, the display is not updated.
+ * If the duration of the playing entry is known, the position slider is
+ * updated along with the elapsed/remaining time display.  Otherwise,
+ * the slider is made insensitive.
+ */
+void
 rb_header_sync_time (RBHeader *header)
 {
-	int seconds;
+	guint seconds;
 
 	if (header->priv->shell_player == NULL)
-		return TRUE;
+		return;
 
 	if (header->priv->slider_dragging == TRUE) {
 		rb_debug ("slider is dragging, not syncing");
-		return TRUE;
+		return;
 	}
 
 	seconds = header->priv->elapsed_time;
 
-	if (header->priv->duration > -1) {
-		double progress = 0.0;
-
-		if (seconds > 0) {
-			progress = (double) (long) seconds;
-		} else {
-			header->priv->adjustment->upper = header->priv->duration;
-			g_signal_emit_by_name (G_OBJECT (header->priv->adjustment), "changed");
-		}
+	if (header->priv->duration > 0) {
+		double progress = (double) seconds;
 
 		header->priv->slider_locked = TRUE;
 		gtk_adjustment_set_value (header->priv->adjustment, progress);
@@ -459,8 +621,6 @@ rb_header_sync_time (RBHeader *header)
 	}
 
 	rb_header_update_elapsed (header);
-
-	return TRUE;
 }
 
 static gboolean
@@ -470,24 +630,25 @@ slider_press_callback (GtkWidget *widget,
 {
 	header->priv->slider_dragging = TRUE;
 	header->priv->latest_set_time = -1;
+	g_object_notify (G_OBJECT (header), "slider-dragging");
+
+	/* HACK: we want the behaviour you get with the middle button, so we
+	 * mangle the event.  clicking with other buttons moves the slider in
+	 * step increments, clicking with the middle button moves the slider to
+	 * the location of the click.
+	 */
+	event->button = 2;
+
+
 	return FALSE;
 }
 
 static gboolean
 slider_moved_timeout (RBHeader *header)
 {
-	double progress;
-	long new;
-
 	GDK_THREADS_ENTER ();
 
-	progress = gtk_adjustment_get_value (gtk_range_get_adjustment (GTK_RANGE (header->priv->scale)));
-	new = (long) (progress+0.5);
-
-	rb_debug ("setting time to %ld", new);
-	rb_shell_player_set_playing_time (header->priv->shell_player, new, NULL);
-
-	header->priv->latest_set_time = new;
+	apply_slider_position (header);
 	header->priv->slider_moved_timeout = 0;
 
 	GDK_THREADS_LEAVE ();
@@ -500,7 +661,6 @@ slider_moved_callback (GtkWidget *widget,
 		       GdkEventMotion *event,
 		       RBHeader *header)
 {
-	GtkAdjustment *adjustment;
 	double progress;
 
 	if (header->priv->slider_dragging == FALSE) {
@@ -508,10 +668,8 @@ slider_moved_callback (GtkWidget *widget,
 		return FALSE;
 	}
 
-	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
-
-	progress = gtk_adjustment_get_value (adjustment);
-	header->priv->elapsed_time = (long) (progress+0.5);
+	progress = gtk_adjustment_get_value (header->priv->adjustment);
+	header->priv->elapsed_time = (guint) (progress+0.5);
 
 	rb_header_update_elapsed (header);
 
@@ -526,54 +684,43 @@ slider_moved_callback (GtkWidget *widget,
 	return FALSE;
 }
 
+static void
+apply_slider_position (RBHeader *header)
+{
+	double progress;
+	long new;
+
+	progress = gtk_adjustment_get_value (header->priv->adjustment);
+	new = (long) (progress+0.5);
+
+	if (new != header->priv->latest_set_time) {
+		rb_debug ("setting time to %ld", new);
+		rb_shell_player_set_playing_time (header->priv->shell_player, new, NULL);
+		header->priv->latest_set_time = new;
+	}
+}
+
 static gboolean
 slider_release_callback (GtkWidget *widget,
 			 GdkEventButton *event,
 			 RBHeader *header)
 {
-	double progress;
-	long new;
-	GtkAdjustment *adjustment;
+	/* HACK: see slider_press_callback */
+	event->button = 2;
 
 	if (header->priv->slider_dragging == FALSE) {
 		rb_debug ("slider is not dragging");
 		return FALSE;
 	}
 
-	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
-
-	progress = gtk_adjustment_get_value (adjustment);
-	new = (long) (progress+0.5);
-
-	if (new != header->priv->latest_set_time) {
-		rb_debug ("setting time to %ld", new);
-		rb_shell_player_set_playing_time (header->priv->shell_player, new, NULL);
-	}
-
-	header->priv->slider_dragging = FALSE;
-
-	rb_header_sync_time (header);
-
 	if (header->priv->slider_moved_timeout != 0) {
 		g_source_remove (header->priv->slider_moved_timeout);
 		header->priv->slider_moved_timeout = 0;
 	}
 
-	return FALSE;
-}
-
-static gboolean
-changed_idle_callback (RBHeader *header)
-{
-	GDK_THREADS_ENTER ();
-
-	slider_release_callback (header->priv->scale, NULL, header);
-
-	header->priv->value_changed_update_handler = 0;
-	rb_debug ("in changed_idle_callback");
-
-	GDK_THREADS_LEAVE ();
-
+	apply_slider_position (header);
+	header->priv->slider_dragging = FALSE;
+	g_object_notify (G_OBJECT (header), "slider-dragging");
 	return FALSE;
 }
 
@@ -581,30 +728,60 @@ static void
 slider_changed_callback (GtkWidget *widget,
 		         RBHeader *header)
 {
+	/* if the slider isn't being dragged, and nothing else is happening,
+	 * this indicates the position was adjusted with a keypress (page up/page down etc.),
+	 * so we should directly apply the change.
+	 */
 	if (header->priv->slider_dragging == FALSE &&
-	    header->priv->slider_locked == FALSE &&
-	    header->priv->value_changed_update_handler == 0) {
-		header->priv->slider_dragging = TRUE;
-		header->priv->value_changed_update_handler =
-			g_idle_add ((GSourceFunc) changed_idle_callback, header);
+	    header->priv->slider_locked == FALSE) {
+		apply_slider_position (header);
 	}
 }
 
+static gboolean
+slider_scroll_callback (GtkWidget *widget, GdkEventScroll *event, RBHeader *header)
+{
+	gboolean retval = TRUE;
+	gdouble adj = gtk_adjustment_get_value (header->priv->adjustment);
+
+	switch (event->direction) {
+	case GDK_SCROLL_UP:
+		rb_debug ("slider scrolling up");
+		gtk_adjustment_set_value (header->priv->adjustment, adj + SCROLL_UP_SEEK_OFFSET);
+		break;
+
+	case GDK_SCROLL_DOWN:
+		rb_debug ("slider scrolling down");
+		gtk_adjustment_set_value (header->priv->adjustment, adj + SCROLL_DOWN_SEEK_OFFSET);
+		break;
+
+	default:
+		retval = FALSE;
+		break;
+	}
+
+	return retval;
+}
+
 static void
 rb_header_update_elapsed (RBHeader *header)
 {
-	char *elapsed_text;
-
 	/* sanity check */
-	if ((header->priv->elapsed_time > header->priv->duration) || (header->priv->elapsed_time < 0))
+	if (header->priv->duration > 0 && header->priv->elapsed_time > header->priv->duration)
 		return;
 
-	elapsed_text = rb_make_elapsed_time_string (header->priv->elapsed_time,
-						    header->priv->duration,
-						    !eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY));
+	if (header->priv->entry != NULL) {
+		char *elapsed_text;
+
+		elapsed_text = rb_make_elapsed_time_string (header->priv->elapsed_time,
+							    header->priv->duration,
+							    !eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY));
+		gtk_label_set_text (GTK_LABEL (header->priv->elapsed), elapsed_text);
+		g_free (elapsed_text);
+	} else {
+		gtk_label_set_text (GTK_LABEL (header->priv->elapsed), "");
+	}
 
-	gtk_label_set_text (GTK_LABEL (header->priv->elapsed), elapsed_text);
-	g_free (elapsed_text);
 }
 
 static void
