@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -119,6 +119,7 @@ rb_search_entry_class_init (RBSearchEntryClass *klass)
 	/**
 	 * RBSearchEntry::activate:
 	 * @entry: the #RBSearchEntry
+	 * @text: search text
 	 *
 	 * Emitted when the entry is activated.
 	 */
@@ -128,9 +129,10 @@ rb_search_entry_class_init (RBSearchEntryClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBSearchEntryClass, activate),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE,
-			      0);
+			      1,
+			      G_TYPE_STRING);
 
 	g_type_class_add_private (klass, sizeof (RBSearchEntryPrivate));
 }
@@ -262,9 +264,11 @@ rb_search_entry_check_style (RBSearchEntry *entry)
 	if (text && *text) {
 		gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, &fg_colour);
 		gtk_widget_modify_base (entry->priv->entry, GTK_STATE_NORMAL, &bg_colour);
+		gtk_widget_modify_cursor (entry->priv->entry, &fg_colour, &fg_colour);
 	} else {
 		gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, NULL);
 		gtk_widget_modify_base (entry->priv->entry, GTK_STATE_NORMAL, NULL);
+		gtk_widget_modify_cursor (entry->priv->entry, NULL, NULL);
 	}
 
 	gtk_widget_queue_draw (GTK_WIDGET (entry));
@@ -338,7 +342,8 @@ static void
 rb_search_entry_activate_cb (GtkEntry *gtkentry,
 			     RBSearchEntry *entry)
 {
-	g_signal_emit (G_OBJECT (entry), rb_search_entry_signals[ACTIVATE], 0);
+	g_signal_emit (G_OBJECT (entry), rb_search_entry_signals[ACTIVATE], 0,
+		       gtk_entry_get_text (GTK_ENTRY (entry->priv->entry)));
 }
 
 /**
